/*
 * Copyright (c) 2005,2006 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 *
 * Startup functions and callback functions called 
 * within main loop is described here.
 *
 * "$Id: main.c,v 1.4 2006/10/09 10:20:22 ryo Exp $"
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef HAVE_SIGNAL_H
#  include <signal.h>
#endif

#include "RLerrno.h"
#include "RLgt.h"
#include "RLlogger.h"
#include "game_manager.h"
#include "spacemaze.h"
#include "state.h"
#include "entity.h"


#define SPACEMAZE_SCREEN_TITLE  "SpaceMaze"
#define DEFAULT_SOUND_DEVICE    "/dev/dsp"

static const char* data_dir = DATA_DIRECTORY;

typedef struct
{
	int   index;
	char* file_name;
	int   use_mask;
}
FileMap;

static FileMap image_file_map[] =
{
	{ INDEX_IMAGE_TITLE,         "title0.png",         RL_GT_USE_MASK },
	{ INDEX_IMAGE_TITLE_INFO,    "title1.png",         RL_GT_USE_MASK },
	{ INDEX_IMAGE_START_NOFOCUS, "start0.png",         RL_GT_USE_MASK },
	{ INDEX_IMAGE_START_FOCUS,   "start1.png",         RL_GT_USE_MASK },
	{ INDEX_IMAGE_START_EFFECT,  "start2.png",         RL_GT_USE_MASK },
	{ INDEX_IMAGE_LEVEL_START,   "start3.png",         RL_GT_USE_MASK },
	{ INDEX_IMAGE_EXIT_NOFOCUS,  "exit0.png",          RL_GT_USE_MASK },
	{ INDEX_IMAGE_EXIT_FOCUS,    "exit1.png",          RL_GT_USE_MASK },
	{ INDEX_IMAGE_PAUSE,         "pause.png",          RL_GT_USE_MASK },
	{ INDEX_IMAGE_TIME0,         "time0.png",          RL_GT_USE_MASK },
	{ INDEX_IMAGE_TIME1,         "time1.png",          RL_GT_USE_MASK },
	{ INDEX_IMAGE_TIME2,         "time2.png",          RL_GT_USE_MASK },
	{ INDEX_IMAGE_TIME3,         "time3.png",          RL_GT_USE_MASK },
	{ INDEX_IMAGE_NOITEM,        "noitem.png",         RL_GT_USE_MASK },
	{ INDEX_IMAGE_BLOCK0,        "block0.png",         RL_GT_NO_MASK  },
	{ INDEX_IMAGE_BLOCK1,        "block1.png",         RL_GT_NO_MASK  },
	{ INDEX_IMAGE_BLOCK2,        "block2.png",         RL_GT_NO_MASK  },
	{ INDEX_IMAGE_BLOCK3,        "block3.png",         RL_GT_NO_MASK  },
	{ INDEX_IMAGE_BLOCK4,        "block4.png",         RL_GT_NO_MASK  },
	{ INDEX_IMAGE_GAMEOVER,      "gameover.png",       RL_GT_USE_MASK },
	{ INDEX_IMAGE_KEY_BLONZE,    "key0.png",           RL_GT_USE_MASK },
	{ INDEX_IMAGE_KEY_SILVER,    "key1.png",           RL_GT_USE_MASK },
	{ INDEX_IMAGE_KEY_GOLD,      "key2.png",           RL_GT_USE_MASK },
	{ INDEX_IMAGE_LEVEL0,        "level0.png",         RL_GT_USE_MASK },
	{ INDEX_IMAGE_LEVEL1,        "level1.png",         RL_GT_USE_MASK },
	{ INDEX_IMAGE_LEVEL2,        "level2.png",         RL_GT_USE_MASK },
	{ INDEX_IMAGE_CLEAR,         "clear.png",          RL_GT_USE_MASK },
	{ INDEX_IMAGE_CONTINUE,      "continue.png",       RL_GT_USE_MASK },
	{ INDEX_IMAGE_GO_BACK,       "goback.png",         RL_GT_USE_MASK },
	{ INDEX_IMAGE_BACK_NOFOCUS,  "back0.png",          RL_GT_USE_MASK },
	{ INDEX_IMAGE_BACK_FOCUS,    "back1.png",          RL_GT_USE_MASK },
	{ INDEX_IMAGE_SAVE_NOFOCUS,  "save0.png",          RL_GT_USE_MASK },
	{ INDEX_IMAGE_SAVE_FOCUS,    "save1.png",          RL_GT_USE_MASK },
	{ INDEX_IMAGE_LOAD_NOFOCUS,  "load0.png",          RL_GT_USE_MASK },
	{ INDEX_IMAGE_LOAD_FOCUS,    "load1.png",          RL_GT_USE_MASK },
	{ INDEX_IMAGE_LOAD_EFFECT,   "load2.png",          RL_GT_USE_MASK },
	{ INDEX_IMAGE_LOAD_FAIL,     "loadfail.png",       RL_GT_USE_MASK },
	{ INDEX_IMAGE_SAVE_DONE,     "savedone.png",       RL_GT_USE_MASK },
	{ INDEX_IMAGE_SAVE_FAIL,     "savefail.png",       RL_GT_USE_MASK },
	{ INDEX_IMAGE_EVACUATE,      "evacuate.png",       RL_GT_USE_MASK },
	{ INDEX_IMAGE_DESTROY,       "destroy.png",        RL_GT_USE_MASK },
	{ INDEX_IMAGE_SAVE_GALAXY,   "savegalaxy.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_EXPLOSION_S,   "explosion_s.png",    RL_GT_USE_MASK },
	{ INDEX_IMAGE_EXPLOSION_M,   "explosion_m.png",    RL_GT_USE_MASK },
	{ INDEX_IMAGE_EXPLOSION_L,   "explosion_l.png",    RL_GT_USE_MASK },
	{ INDEX_IMAGE_EXPLOSION_G,   "explosion_g.png",    RL_GT_USE_MASK },
	{ INDEX_IMAGE_COPYRIGHT,     "copyright.png",      RL_GT_USE_MASK },
	{ INDEX_IMAGE_CONGRATULATE,  "congratulation.png", RL_GT_USE_MASK },
	{ INDEX_IMAGE_THANKYOU,      "thankyou.png",       RL_GT_USE_MASK },
	{ INDEX_IMAGE_TOTALTIME,     "totaltime.png",      RL_GT_USE_MASK },
	{ INDEX_IMAGE_RANK,          "rank.png",           RL_GT_USE_MASK },
	{ INDEX_IMAGE_RANK_S,        "rank_s.png",         RL_GT_USE_MASK },
	{ INDEX_IMAGE_RANK_A,        "rank_a.png",         RL_GT_USE_MASK },
	{ INDEX_IMAGE_RANK_B,        "rank_b.png",         RL_GT_USE_MASK },
	{ INDEX_IMAGE_RANK_C,        "rank_c.png",         RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS0_0,    "numbers0_0.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS0_1,    "numbers0_1.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS0_2,    "numbers0_2.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS0_3,    "numbers0_3.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS0_4,    "numbers0_4.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS0_5,    "numbers0_5.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS0_6,    "numbers0_6.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS0_7,    "numbers0_7.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS0_8,    "numbers0_8.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS0_9,    "numbers0_9.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS0_DM,   "numbers0_dm.png",    RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS1_0,    "numbers1_0.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS1_1,    "numbers1_1.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS1_2,    "numbers1_2.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS1_3,    "numbers1_3.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS1_4,    "numbers1_4.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS1_5,    "numbers1_5.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS1_6,    "numbers1_6.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS1_7,    "numbers1_7.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS1_8,    "numbers1_8.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS1_9,    "numbers1_9.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS1_DM,   "numbers1_dm.png",    RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS2_0,    "numbers2_0.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS2_1,    "numbers2_1.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS2_2,    "numbers2_2.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS2_3,    "numbers2_3.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS2_4,    "numbers2_4.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS2_5,    "numbers2_5.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS2_6,    "numbers2_6.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS2_7,    "numbers2_7.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS2_8,    "numbers2_8.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS2_9,    "numbers2_9.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS2_DM,   "numbers2_dm.png",    RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS3_0,    "numbers3_0.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS3_1,    "numbers3_1.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS3_2,    "numbers3_2.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS3_3,    "numbers3_3.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS3_4,    "numbers3_4.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS3_5,    "numbers3_5.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS3_6,    "numbers3_6.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS3_7,    "numbers3_7.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS3_8,    "numbers3_8.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS3_9,    "numbers3_9.png",     RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS3_DM,   "numbers3_dm.png",    RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS0,      "numbers0.png",       RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS1,      "numbers1.png",       RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS2,      "numbers2.png",       RL_GT_USE_MASK },
	{ INDEX_IMAGE_NUMBERS3,      "numbers3.png",       RL_GT_USE_MASK },
	{ -1,                        NULL,                 RL_GT_NO_MASK  }
};


static FileMap wave_file_map[] =
{
	{ INDEX_WAVE_CLICK,     "click.wav",     -1 },
	{ INDEX_WAVE_ESC,       "esc.wav",       -1 },
	{ INDEX_WAVE_PAUSE,     "pause.wav",     -1 },
	{ INDEX_WAVE_SELECT,    "select.wav",    -1 },
	{ INDEX_WAVE_CLEAR,     "clear.wav",     -1 },
	{ INDEX_WAVE_START,     "start.wav",     -1 },
	{ INDEX_WAVE_GETKEY,    "getkey.wav",    -1 },
	{ INDEX_WAVE_DAMAGE,    "damage.wav",    -1 },
	{ INDEX_WAVE_DOUGHNUT,  "doughnut.wav",  -1 },
	{ INDEX_WAVE_EXPLOSION, "explosion.wav", -1 },
	{ INDEX_WAVE_MISSILE,   "missile.wav",   -1 },
	{ INDEX_WAVE_SHOT,      "shot.wav",      -1 },
	{ INDEX_WAVE_SUSPEND,   "suspend.wav",   -1 },
	{ -1,                   NULL,            -1 }
};


static void usage( void );
static void init( int argc, char** argv );
static void load_images( void );
static void load_waves( void );
static void update( RLgtApp* app );
static void draw( RLgtApp* app );
static void cleanup( void* data );


static SpaceMazeApp spacemaze_app;

#ifdef HAVE_SIGNAL_H
static void set_signal_handler( void );
static void signal_handler( int signum );
#endif


/*!
 * Entry point.
 */
int 
main( int argc, char** argv )
{
	int exitcode;

	init( argc, argv );

	rlib_gt_enable_dropframe( &( spacemaze_app.gt ) );
	rlib_gt_set_update_func( &( spacemaze_app.gt ), update );
	rlib_gt_set_draw_func( &( spacemaze_app.gt ), draw );
	rlib_gt_set_cleanup_func( &( spacemaze_app.gt ), cleanup, &spacemaze_app );

#ifdef HAVE_SIGNAL_H
	set_signal_handler();
#endif

	rlib_gt_start( &( spacemaze_app.gt ) );
	rlib_gt_get_exitcode( &( spacemaze_app.gt ), &exitcode );

	return exitcode;
}


static void
usage( void )
{
	printf( "\n" );
	printf( "Usage: spacemaze [options]...\n" );
	printf( "\n" );
	printf( "Options:\n" );
	printf( "\n" );
	printf( "--help                      Shows help messages.\n" );
	printf( "--enable-sound              Enables sound function.\n" );
	printf( "--sound-device=DEVICE_NAME  Specify sound device file ('/dev/dsp' as default).\n" );
	printf( "--fps=FPS                   Specify frame rate between 30 - 60 (45 as default).\n" );
	printf( "\n" );
	printf( "See manual page to understand how to play this game.\n" );
	printf( "\n" );
}


static void
init( int argc, char** argv )
{
	char* opt_help         = "--help";
	char* opt_debug        = "--debug";
	char* opt_sound_device = "--sound-device=";
	char* opt_enable_sound = "--enable-sound";
	char* opt_fps          = "--fps=";

	char* device_name  = DEFAULT_SOUND_DEVICE;
	int   enable_sound = 0;
	int   fps = SPACEMAZE_DEFAULT_FPS;
	int   i;

	SRAND();

	memset( &spacemaze_app, 0, sizeof( SpaceMazeApp ) );
	spacemaze_app.title_screen.is_start_focused = 1;

	if ( argc > 1 ) {
		for ( i = 1; i < argc; i ++ ) {
			if ( strcmp( argv[i], opt_debug ) == 0 ) {
				spacemaze_app.is_debug_on = 1;
			}
			if ( strcmp( argv[i], opt_help ) == 0 ) {
				usage();
				exit( EXIT_SUCCESS );
			}
			else if ( strcmp( argv[i], opt_enable_sound ) == 0 ) {
				enable_sound = 1;
			}
			else if ( strncmp( opt_sound_device, argv[i], strlen( opt_sound_device ) ) == 0 ) {
				if ( strlen( argv[i] ) > strlen( opt_sound_device ) ) {
					device_name = &( argv[i][ strlen( opt_sound_device ) ] );
				}
			}
			else if ( strncmp( opt_fps, argv[i], strlen( opt_fps ) ) == 0 ) {
				if ( strlen( argv[i] ) > strlen( opt_fps ) ) {
					fps = atoi( &( argv[i][ strlen( opt_fps ) ] ) );

					if ( fps < SPACEMAZE_MIN_FPS || fps > SPACEMAZE_MAX_FPS ) {
						fps = SPACEMAZE_DEFAULT_FPS;
					}
				} 
			}
		}
	}

	
	if ( rlib_gt_initialize( &( spacemaze_app.gt ), SPACEMAZE_SCREEN_TITLE, -1, -1, 
		 SPACEMAZE_SCREEN_WIDTH, SPACEMAZE_SCREEN_HEIGHT ) != 0 ) {
		rlib_logger_fatal( "Failed to initialize graphics toolkit ( rlib_gt_initialize() )." );
		exit( EXIT_FAILURE );
	}

	rlib_gt_set_fps( &( spacemaze_app.gt ), fps );
	state_set_fps_ratio( ( ( double )fps / ( double )SPACEMAZE_MAX_FPS ) );
	entity_set_fps_ratio( ( ( double )fps / ( double )SPACEMAZE_MAX_FPS ) );

	load_images();

	if ( enable_sound ) {
		rlib_sound_initialize( &( spacemaze_app.sound ), device_name );
		load_waves();
	}

	entity_set_resource_indexes( spacemaze_app.image_index, spacemaze_app.wave_index );
	game_manager_init( &spacemaze_app );
}


static void
load_images( void )
{
	char file_name[512];
	int  i = 0;
	int  index;

	for ( ;; i ++ ) {
		if ( image_file_map[i].index == -1 ) {
			break;
		}

		snprintf( file_name, sizeof( file_name ), "%s/png/%s", data_dir, image_file_map[i].file_name );
		index = rlib_gt_load_image( &( spacemaze_app.gt ), file_name, image_file_map[i].use_mask );

		if ( index == RLIB_ERR || index == RLIB_EFAULT || index == RLIB_ENOSPC ) {
			char logbuf[512];
			snprintf( logbuf, sizeof( logbuf ), 
				"Failed to load image file %s, errno = %d", file_name, index );
			rlib_logger_fatal( logbuf );
			rlib_gt_cleanup( &( spacemaze_app.gt ) );
			exit( EXIT_FAILURE );
		}

		spacemaze_app.image_index[ image_file_map[i].index ] = index;
	}
}


static void
load_waves( void )
{
	char file_name[512];
	int  i = 0;
	int  index;

	for ( ;; i ++ ) {
		if ( wave_file_map[i].index == -1 ) {
			break;
		}

		snprintf( file_name, sizeof( file_name ), "%s/wave/%s", data_dir, wave_file_map[i].file_name );
		index = rlib_sound_load_wave( &( spacemaze_app.sound ), file_name );

		if ( index == RLIB_ERR || index == RLIB_EFAULT || index == RLIB_ENOSPC ) {
			char logbuf[512];
			snprintf( logbuf, sizeof( logbuf ), 
				"Failed to load wave file %s, errno = %d", file_name, index );
			rlib_logger_fatal( logbuf );
			rlib_sound_shutdown( &( spacemaze_app.sound ) );
			rlib_gt_cleanup( &( spacemaze_app.gt ) );
			exit( EXIT_FAILURE );
		}

		spacemaze_app.wave_index[ wave_file_map[i].index ] = index;
	}
}


#ifdef HAVE_SIGNAL_H
static void
set_signal_handler( void )
{
	struct sigaction sact;

	memset( &sact, 0, sizeof( sact ) );
	sact.sa_handler = signal_handler;

	sigaction( SIGINT,  &sact, NULL );
	sigaction( SIGPIPE, &sact, NULL );
	sigaction( SIGQUIT, &sact, NULL );
	sigaction( SIGTERM, &sact, NULL );
}


static void
signal_handler( int signum )
{
	sigset_t mask;
	char logbuf[32];

	sigemptyset( &mask );
	sigaddset( &mask, SIGINT );
	sigaddset( &mask, SIGPIPE );
	sigaddset( &mask, SIGQUIT );
	sigaddset( &mask, SIGTERM );
	sigprocmask( SIG_BLOCK, &mask, NULL );

	snprintf( logbuf, sizeof( logbuf ), "Caught signal %d, going to exit.", signum );
	rlib_logger_info( logbuf );
	rlib_sound_shutdown( &( spacemaze_app.sound ) );
	rlib_gt_cleanup( &( spacemaze_app.gt ) );
	exit( EXIT_SUCCESS );
}
#endif


static void
update( RLgtApp* app )
{
	if ( spacemaze_app.is_debug_on ) {
		if ( rlib_gt_key_pressed( app, RL_GT_KEY_h ) ) {
			if ( rlib_gt_is_debug_on( app ) ) {
				rlib_gt_debug_off( app );
			}
			else {
				rlib_gt_debug_on( app );
			}
		}

		if ( rlib_gt_key_pressed( app, RL_GT_KEY_f ) ) {
			if ( rlib_gt_is_dropframe_enabled( app ) ) {
				rlib_gt_disable_dropframe( app );
			}
			else {
				rlib_gt_enable_dropframe( app );
			}
		}

		if ( rlib_gt_mouse_pressed( app, RL_GT_MOUSE_LEFT, NULL, NULL ) ) {
			DUMP_CONTEXT( &spacemaze_app );
		}
	}

	game_manager_update( &spacemaze_app );
}


static void
draw( RLgtApp* app )
{
	game_manager_draw( &spacemaze_app );
}


static void
cleanup( void* data )
{
	/* NOP */
}


/* End of main.c */
