/*
 * Copyright (c) 2005,2006 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief Handles enemy (ball) data and draws it onto the screen.
 *
 * "$Id: ball.c,v 1.1.1.1 2006/07/22 18:07:56 ryo Exp $"
 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "RLgt.h"
#include "RLrect.h"
#include "enemy.h"
#include "entity.h"
#include "explosion.h"
#include "missile.h"
#include "spaceship.h"


#define FRAME_COUNT_TO_NEXT_LAUNCH 150

#define BALL_WIDTH                 40
#define BALL_HEIGHT                40
#define BALL_LAUNCH_DISTANCE       150
#define BALL_MAX_ENERGY            50
#define BALL_STRENGTH              7
#define BALL_VELOCITY              0.8


static void ball_draw_body( Entity* ball );
static void ball_launch_missile( Entity* ball, Entity* spaceship );


/*!
 * Initializes enemy (ball) specific data.
 * No null check is executed because this is a back-end of enemy_init().
 */
void
ball_init( Entity* ball )
{
	double angle = RANDOM( 2.0 * PI );

	ball->width        = BALL_WIDTH;
	ball->height       = BALL_HEIGHT;
	ball->energy       = BALL_MAX_ENERGY;
	ball->strength     = BALL_STRENGTH;
	ball->vmax         = ( BALL_VELOCITY / entity_get_fps_ratio() );
	ball->vx           = ( BALL_VELOCITY / entity_get_fps_ratio() ) * cos( angle );
	ball->vy           = ( BALL_VELOCITY / entity_get_fps_ratio() ) * sin( angle );
	ball->check_func   = ball_check;
	ball->update_func  = ball_update;
	ball->draw_func    = ball_draw;
}


/*!
 * Computes new position just for collision check.
 * Note that this function does not compute real position.
 * After this, execute entity_update() to compute real position.
 * This function returns 0 when no collision check is needed 
 * because enemy is not alive ...
 *
 * No null check is executed because this is a back-end of entity_check().
 */
int
ball_check( Entity* ball, RLrect* rect, double* vx, double* vy )
{
	if ( ! entity_is_alive( ball ) ) {
		return 0;
	}

	rect->lefttop_x = ball->px + ball->vx - ( ball->width  * 0.45 );
	rect->lefttop_y = ball->py + ball->vy - ( ball->height * 0.45 );
	rect->width     = ( int )( ball->width  * 0.9 );
	rect->height    = ( int )( ball->height * 0.9 );

	(*vx) = ball->vx;
	(*vy) = ball->vy;

	return 1;
}


/*!
 * Commits new position, status, etc. at the next frame.
 * No null check is executed because this is a back-end of entity_update().
 * spaceship can be NULL, which will be ignored in this function.
 */
void
ball_update( Entity* ball, Entity* spaceship )
{
	if ( ball->is_exploding ) {
		if ( spaceship != NULL ) {
			explosion_update( &( ball->explosion ), spaceship->px, spaceship->py );
		}
		else {
			explosion_update( &( ball->explosion ), -1, -1 );
		}

		if ( ! explosion_is_exploding( &( ball->explosion ) ) ) {
			ball->is_alive     = 0;
			ball->is_exploding = 0;
		}
	}
	else if ( ball->is_alive ) {
		if ( ball->frames_to_next_launch > 0 ) {
			ball->frames_to_next_launch --;
		}

		if ( spaceship != NULL ) {
			ball_launch_missile( ball, spaceship );
		}

		ball->px += ball->vx;
		ball->py += ball->vy;
	}
}


static void
ball_launch_missile( Entity* ball, Entity* spaceship )
{
	if ( spaceship == NULL ) {
		return;
	}

	{
		double mx = ball->px;
		double my = ball->py;
		double px;
		double py;
		double distance;

		entity_get_position( spaceship, &px, &py );
		distance = DISTANCE( ball->px, ball->py, px, py );

		if ( distance < BALL_LAUNCH_DISTANCE && ball->frames_to_next_launch <= 0 ) {
			ball->frames_to_next_launch = 
				( FRAME_COUNT_TO_NEXT_LAUNCH * entity_get_fps_ratio() );
			missile_launch_three_way( ball->gt_app, ball->missiles, MISSILE_MINE,
				&( ball->screen ), mx, my, px - mx, py - my );
		}
	}
}


/*!
 * Draws an enemy (ball) onto the screen.
 * No null check is executed because this is a back-end of entity_draw().
 */
void
ball_draw( Entity* ball )
{
	if ( entity_is_exploding( ball ) ) {
		explosion_draw( &( ball->explosion ) );
		return;
	}

	if ( ball->is_alive ) {
		ball_draw_body( ball );
	}
}


static void
ball_draw_body( Entity* ball )
{
#define BALL_BODY_POINTS_NUM 9

	const double points[ BALL_BODY_POINTS_NUM ][2] = {
		{  10.0,  20.0 },
		{  20.0,  10.0 },
		{  20.0, -10.0 },
		{  10.0, -20.0 },
		{ -10.0, -20.0 },
		{ -20.0, -10.0 },
		{ -20.0,  10.0 },
		{ -10.0,  20.0 },
		{  10.0,  20.0 }
	};

	Point converted[ BALL_BODY_POINTS_NUM ];
	int screen_x;
	int screen_y;
	int culling = 1;
	int i;

	SCREEN_POSITION( ball, screen_x, screen_y );

	for ( i = 0; i < BALL_BODY_POINTS_NUM; i ++ ) {
		converted[i].x = ( short )( screen_x + points[i][0] );
		converted[i].y = ( short )( screen_y + points[i][1] );

		if ( rlib_rect_within( &( ball->screen ), converted[i].x, converted[i].y ) ) {
			culling = 0;
		}
	}

	if ( culling ) {
		return;
	}

	rlib_gt_set_foreground_color( ball->gt_app, 0, 0, RL_GT_MAX_RGB );
	rlib_gt_draw_lines( ball->gt_app, converted, BALL_BODY_POINTS_NUM );
	rlib_gt_set_foreground_color( ball->gt_app, RL_GT_MAX_RGB, 0, 0 );

	rlib_gt_fill_arc( 
		ball->gt_app,
		screen_x - ball->width  / 5.0, 
		screen_y - ball->height / 5.0,
		ball->width  / 5.0,
		ball->height / 5.0,
		0,
		360 * 64
		);
}


/* End of ball.c */
