/*
 * Copyright (c) 2005,2006 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief Header file defines common parameters and 
 * data structures of various kinds of entity.
 *
 * "$Id: entity.h,v 1.1.1.1 2006/07/22 18:07:56 ryo Exp $"
 */

#ifndef ___ENTITY__H_INCLUDED_
#define ___ENTITY__H_INCLUDED_

#include <stdlib.h>

#include "RLgt.h"
#include "RLlist.h"
#include "RLrect.h"
#include "RLsound.h"
#include "explosion.h"


#define THRESHOLD 0.01
#define PI 3.14


/*!
 * @brief Data structure represents an entity.
 */
typedef struct _Entity
{
	RLgtApp*        gt_app;
	RLsoundContext* sound;
	RLrect          screen;
	RLlist*         missiles;
	Explosion       explosion;

	int             effect_animation_frame_count;
	int             effect_blink_frame_count;
	int             frames_to_next_launch;
	int             drawing_image;
	int*            image_index;
	int*            wave_index;

	int             is_alive;
	int             is_exploding;
	int             is_thrusting;
	int             is_back_thrusting;

	int             mode;
	int             key_released;

	int             width;
	int             height;
	int             energy;
	int             strength;
	double          vmax;

	/*
	 * Following values are the ones correspond to 
	 * the world x-y coordinates.
	 */
	double          screen_center_x;
	double          screen_center_y;
	double          initial_px;
	double          initial_py;
	double          px;
	double          py;
	double          vx;
	double          vy;
	double          ax;
	double          ay;
	double          prot;
	double          vrot;
	double          arot;

	void ( *decrease_func )( struct _Entity*, int );
	void ( *explode_func )( struct _Entity* );
	void ( *blink_func )( struct _Entity* );
	void ( *set_velocity_func )( struct _Entity*, double, double );
	int  ( *key_released_func )( struct _Entity* );
	int  ( *check_func )( struct _Entity*, RLrect*, double*, double* );
	void ( *update_func )( struct _Entity*, struct _Entity* );
	void ( *draw_func )( struct _Entity* );
}
Entity;


void   entity_set_fps_ratio( double ratio );
double entity_get_fps_ratio();
int    entity_get_framecount_effect_blink();
int    entity_get_framecount_effect_animation();
void   entity_set_resource_indexes( int* image_index, int* wave_index );
void   entity_init( Entity* entity, RLgtApp* app, RLsoundContext* sound, RLrect* screen );
int    entity_is_alive( Entity* entity );
int    entity_is_exploding( Entity* entity );
void   entity_explode( Entity* entity );
int    entity_get_strength( Entity* entity );
int    entity_get_energy( Entity* entity );
void   entity_decrease_energy( Entity* entity, int amount );
void   entity_blink( Entity* entity );
void   entity_set_missile_list( Entity* entity, RLlist* missiles );
int    entity_get_width( Entity* entity );
int    entity_get_height( Entity* entity );
void   entity_get_position( Entity* entity, double* x, double * y );
void   entity_set_position( Entity* entity, double x, double y );
void   entity_get_velocity( Entity* entity, double* vx, double* vy );
void   entity_set_velocity( Entity* entity, double vx, double vy );
double entity_get_rotation( Entity* entity );
void   entity_set_rotation( Entity* entity, double prot );
int    entity_key_released( Entity* entity );
int    entity_check( Entity* entity, RLrect* rect, double* vx, double* vy );
void   entity_update( Entity* entity, Entity* spaceship );
void   entity_draw( Entity* entity );
int    entity_calculate_reflection( Entity* entity1, Entity* entity2 );


#define RANDOM( v ) ( ( double )(v) * rand() / ( RAND_MAX + 1.0 ) )
#define DISTANCE( x1, y1, x2, y2 ) ( sqrt( pow( (x1) - (x2), 2 ) + pow( (y1) - (y2), 2 ) ) )
#define SIZE( vec_x, vec_y ) ( sqrt( pow( ( vec_x ), 2 ) + pow( ( vec_y ), 2 ) ) )

#define NORMALIZE( vec_x, vec_y ) { \
        double size = sqrt( pow( (vec_x), 2 ) + pow( (vec_y), 2 ) ); \
        if ( size >= THRESHOLD ) { \
                (vec_x) /= size; \
                (vec_y) /= size; \
        } \
        else { \
                (vec_x) = (vec_y) = 0; \
        } \
}

#define SCREEN_POSITION( entity, x, y ) { \
        (x) = (entity)->screen.lefttop_x + (entity)->screen.width / 2  + \
                (entity)->px - (entity)->screen_center_x; \
        (y) = (entity)->screen.lefttop_y + (entity)->screen.height / 2 + \
                (entity)->py - (entity)->screen_center_y; \
}


#endif /* ___ENTITY_H_INCLUDED_ */

/* End of entity.h */
