/*
 * Copyright (c) 2006 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief Handles explosion entity and draws it onto the screen.
 *
 * "$Id: explosion.c,v 1.1.1.1 2006/07/22 18:07:56 ryo Exp $"
 */

#include <stdio.h>

#include "RLgt.h"
#include "explosion.h"
#include "resource.h"


/*!
 * Initializes explosion data structure.
 *
 * @param explosion   [i/o] An explosion data storage.
 * @param app         [in]  GUI toolkit context.
 * @param screen      [in]  Screen definition on which the explosion should be drawn.
 * @param image_index [in]  Image indexes to refer to the real data stored in GUI toolkit context.
 */
void
explosion_init( Explosion* explosion, RLgtApp* app, RLrect* screen, int* image_index )
{
	if ( explosion == NULL || app == NULL || screen == NULL || image_index == NULL ) {
		return;
	}

	memset( explosion, 0, sizeof( Explosion ) );

	explosion->gt_app           = app;
	explosion->is_exploding     = 1;
	explosion->image_index      = image_index;
	explosion->screen.lefttop_x = screen->lefttop_x;
	explosion->screen.lefttop_y = screen->lefttop_y;
	explosion->screen.width     = screen->width;
	explosion->screen.height    = screen->height;
}


/*!
 * Checks if specified explosion is exploding or not.
 * Returns 1 when alive, 0 when not or entity is NULL.
 */
int
explosion_is_exploding( Explosion* explosion )
{
	if ( explosion == NULL ) {
		return 0;
	}

	return explosion->is_exploding;
}


/*!
 * Commits new position, status, etc. at the next frame.
 */
void
explosion_update( Explosion* explosion, double center_x, double center_y )
{
	if ( explosion == NULL ) {
		return;
	}

	if ( explosion_is_exploding( explosion ) ) {
		explosion->effect_explosion_frame_count ++;

		if ( explosion->effect_explosion_frame_count >= EXPLOSION_FRAME_COUNT_EFFECT ) {
			explosion->is_exploding = 0;
			explosion->effect_explosion_frame_count = 0;
		}
	}

	if ( center_x != -1 && center_y != -1 ) {
		explosion->screen_center_x = center_x;
		explosion->screen_center_y = center_y;
	}
}


/*!
 * Draws an explosion onto the game screen.
 */
void
explosion_draw( Explosion* explosion )
{
	if ( explosion == NULL || explosion->gt_app == NULL ) {
		return;
	}

	if ( explosion_is_exploding( explosion ) ) {
		int count = explosion->effect_explosion_frame_count;
		int image_index;
		int screen_x;
		int screen_y;

		EXPLOSION_SCREEN_POSITION( explosion, screen_x, screen_y );

		if ( count > EXPLOSION_FRAME_COUNT_EFFECT * 3.0 / 4.0 ) {
			image_index = explosion->image_index[ INDEX_IMAGE_EXPLOSION_G ];
		}
		else if ( count > EXPLOSION_FRAME_COUNT_EFFECT / 2.0 ) {
			image_index = explosion->image_index[ INDEX_IMAGE_EXPLOSION_L ];
		}
		else if ( count > EXPLOSION_FRAME_COUNT_EFFECT / 4.0 ) {
			image_index = explosion->image_index[ INDEX_IMAGE_EXPLOSION_M ];
		}
		else {
			image_index = explosion->image_index[ INDEX_IMAGE_EXPLOSION_S ];
		}

		RL_GT_DRAW_IMAGE( explosion->gt_app, image_index, screen_x, screen_y );
	}
}


/*!
 * Sets the position of an explosion correspond to the world coordinates.
 */
void
explosion_set_position( Explosion* explosion, double x, double y )
{
	if ( explosion == NULL ) {
		return;
	}

	explosion->px = x;
	explosion->py = y;
}


/* End of explosion.c */
