/*
 * Copyright (c) 2005,2006 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief Handles map-data and draws it onto the screen.
 *
 * "$Id: map.c,v 1.1.1.1 2006/07/22 18:07:56 ryo Exp $"
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "RLgt.h"
#include "RLlist.h"
#include "RLlogger.h"
#include "RLrect.h"
#include "RLsound.h"
#include "enemy.h"
#include "item.h"
#include "map.h"
#include "map_util.h"
#include "spaceship.h"
#include "resource.h"


#define MAP_BLOCK_WIDTH        40
#define MAP_BLOCK_HEIGHT       40
#define MAP_BLOCK_WIDTH_RADAR  5
#define MAP_BLOCK_HEIGHT_RADAR 5

#define MAP_AVAILABLE          0
#define MAP_NOT_AVAILABLE      1


static const char* data_dir = DATA_DIRECTORY;

typedef struct
{
	double center_x;
	double center_y;
	double block_w;
	double block_h;
	int lt_row;
	int lt_col;
	int rb_row;
	int rb_col;
}
MapRegion;


static int  map_get_region( Map* map, RLrect* screen, MapRegion* region );
static void map_setup_spaceship( Map* map, Entity* spaceship, int row, int col );
static void map_setup_enemy( Map* map, RLlist* enemies, EnemyType type, int row, int col );
static void map_setup_key( Map* map, Entity* key, int index, int row, int col, int level );

static void map_get_region_for_collision_check( 
	RLrect* rect, double* vx, double* vy,
	int* min_row, int* max_row, int* min_col, int* max_col );

static void map_recompute_velocity( Map* map, RLrect* rect, 
    int block_row, int block_col, double* vx, double* vy );

static int  map_is_block( unsigned char map_element );
static int  map_is_normal_block( unsigned char map_element );
static int  map_is_key_lock( unsigned char map_element );


/*!
 * Initializes map-data structure.
 *
 * @param map          [i/o] The map data storage.
 * @param app          [in]  GUI toolkit context.
 * @param sound        [in]  Sound library context.
 * @param screen       [in]  Screen definition on which map should be drawn.
 * @param radar_screen [in]  Screen definition on which radar map should be drawn.
 * @param image_index  [in]  Image indexes to refer to the real data stored in GUI toolkit context.
 */
void
map_init( Map* map, RLgtApp* app, RLsoundContext* sound, RLrect* screen, RLrect* radar_screen, int* image_index )
{
	if ( map == NULL || app == NULL || sound == NULL ||
	     screen == NULL || radar_screen == NULL || image_index == NULL ) {
		return;
	}

	memset( map, 0, sizeof( Map ) );

	map->gt_app      = app;
	map->sound       = sound;
	map->center_x    = 0.0;
	map->center_y    = 0.0;
	map->image_index = image_index;

	RLRECT_COPY( &( map->screen ), screen );
	RLRECT_COPY( &( map->radar_screen ), radar_screen );
}


/*!
 * Gets the position of the space ship and holds it in the map entity.
 * This position represents the center of the screen.
 */
void
map_update( Map* map, Entity* spaceship )
{
	if ( map == NULL || spaceship == NULL ) {
		return;
	}

	entity_get_position( spaceship, &( map->center_x ), &( map->center_y ) );
}


/*!
 * Draws map-data onto the screen.
 */
void
map_draw( Map* map )
{
	MapRegion region;
	int i;
	int j;

	if ( map == NULL || map->gt_app == NULL ) {
		return;
	}

	region.center_x = map->center_x;
	region.center_y = map->center_y;
	region.block_w  = MAP_BLOCK_WIDTH;
	region.block_h  = MAP_BLOCK_HEIGHT;

	if ( ! map_get_region( map, &( map->screen ), &region ) ) {
		return;
	}

	for ( i = region.lt_row; i <= region.rb_row; i ++ ) {
		for ( j = region.lt_col; j <= region.rb_col; j ++ ) {

			if ( ! map_is_block( map->data[i][j] ) ) {
				continue;
			}

			{
				int image_w;
				int image_h;
				int index;
				int lt_x;
				int lt_y;

				if ( map_is_normal_block( map->data[i][j] ) ) {
					index = map->image_index[ INDEX_IMAGE_BLOCK0 +  map->level ];
				}
				else if ( map_is_entrance_lock( map->data[i][j] ) && map->show_entrance_lock ) {
					index = map->image_index[ INDEX_IMAGE_BLOCK4 ];
				}
				else if ( map_is_key_lock( map->data[i][j] ) && map->show_key_lock ) {
					index = map->image_index[ INDEX_IMAGE_BLOCK4 ];
				}
				else {
					continue;
				}

				lt_x  = ( int )( map->screen.width  / 2 + 
						 ( MAP_BLOCK_WIDTH  * j - region.center_x ) );
				lt_y  = ( int )( map->screen.height / 2 + 
						 ( MAP_BLOCK_HEIGHT * i - region.center_y ) );

				rlib_gt_get_image_size( map->gt_app, index, &image_w, &image_h );
				rlib_gt_draw_image( map->gt_app, index, 0, 0, image_w, image_h, lt_x, lt_y );
			}
		}
	}
}


/*!
 * Draws map-data onto the radar screen.
 */
void
map_draw_radar( Map* map )
{
	MapRegion region;
	int i;
	int j;
	int radar_center_x;
	int radar_center_y;
	int x;
	int y;

	if ( map == NULL || map->gt_app == NULL ) {
		return;
	}

	region.center_x = ( map->center_x / MAP_BLOCK_WIDTH  ) * MAP_BLOCK_WIDTH_RADAR;
	region.center_y = ( map->center_y / MAP_BLOCK_HEIGHT ) * MAP_BLOCK_HEIGHT_RADAR;
	region.block_w  = MAP_BLOCK_WIDTH_RADAR;
	region.block_h  = MAP_BLOCK_HEIGHT_RADAR;

	if ( ! map_get_region( map, &( map->radar_screen ), &region ) ) {
		return;
	}

	radar_center_x = map->radar_screen.lefttop_x + map->radar_screen.width  / 2;
	radar_center_y = map->radar_screen.lefttop_y + map->radar_screen.height / 2;

	if ( map->show_radar_center ) {
		rlib_gt_set_foreground_color( map->gt_app, RL_GT_MAX_RGB, RL_GT_MAX_RGB, RL_GT_MAX_RGB );
		rlib_gt_draw_point( map->gt_app, radar_center_x, radar_center_y );
	}

	rlib_gt_set_foreground_color( map->gt_app, 0, 0, RL_GT_MAX_RGB );
	for ( i = region.lt_row; i <= region.rb_row; i ++ ) {
		for ( j = region.lt_col; j <= region.rb_col; j ++ ) {

			if ( ! map_is_normal_block( map->data[i][j] ) ) {
				continue;
			}

			x = ( int )( radar_center_x + ( MAP_BLOCK_WIDTH_RADAR  * j - region.center_x ) );
			y = ( int )( radar_center_y + ( MAP_BLOCK_HEIGHT_RADAR * i - region.center_y ) );
			rlib_gt_draw_point( map->gt_app, x, y );
		}
	}

	rlib_gt_set_foreground_color( map->gt_app, RL_GT_MAX_RGB, 0, 0 );

	for ( i = region.lt_row; i <= region.rb_row; i ++ ) {
		for ( j = region.lt_col; j <= region.rb_col; j ++ ) {

			if ( ! map_is_entrance_lock( map->data[i][j] ) ) {
				continue;
			}

			x = ( int )( radar_center_x + ( MAP_BLOCK_WIDTH_RADAR  * j - region.center_x ) );
			y = ( int )( radar_center_y + ( MAP_BLOCK_HEIGHT_RADAR * i - region.center_y ) );
			rlib_gt_draw_point( map->gt_app, x, y );
		}
	}
}


static int
map_get_region( Map* map, RLrect* screen, MapRegion* region )
{
	region->lt_row = ( region->center_y - screen->height / 2 ) / region->block_h;
	region->lt_col = ( region->center_x - screen->width  / 2 ) / region->block_w;
	region->rb_row = ( region->center_y + screen->height / 2 ) / region->block_h;
	region->rb_col = ( region->center_x + screen->width  / 2 ) / region->block_w;

	if ( region->lt_row > MAP_ROW_NUM - 1 || region->rb_row < 0 ) {
		return 0;
	}

	if ( region->lt_col > MAP_COL_NUM - 1 || region->rb_col < 0 ) {
		return 0;
	}

	region->lt_row = ( region->lt_row < 0 ) ? 0 : region->lt_row;
	region->lt_col = ( region->lt_col < 0 ) ? 0 : region->lt_col;
	region->rb_row = ( region->rb_row > MAP_ROW_NUM - 1 ) ? MAP_ROW_NUM - 1 : region->rb_row;
	region->rb_col = ( region->rb_col > MAP_COL_NUM - 1 ) ? MAP_COL_NUM - 1 : region->rb_col;

	return 1;
}


/*!
 * Loads a map-data file of the specified level.
 * If a map-data file of the specified level cannot be found, 
 * returns MAP_NOT_AVAILABLE. Returns MAP_AVAILABLE 
 * if successfully processed.
 */
int
map_load( Map* map, Entity* spaceship, RLlist* enemies, Entity* key, int level )
{
	unsigned char  buf[ MAP_FILE_SIZE ];
	unsigned char* bufp;
	char file_name[512];
	int  i;
	int  j;

	if ( map == NULL || map->gt_app == NULL || spaceship == NULL || 
	     enemies == NULL || key == NULL ) {
		return MAP_NOT_AVAILABLE;
	}

	snprintf( file_name, sizeof( file_name ), "%s/map/map%d.dat", data_dir, level );

	if ( map_check_and_load_datafile( file_name, buf ) != MAP_AVAILABLE ) {
		rlib_gt_stop( map->gt_app, EXIT_FAILURE );
		return MAP_NOT_AVAILABLE;
	}

	bufp = buf + MAP_HEAD_SIZE;
	
	for ( i = 0; i < MAP_ROW_NUM; i ++, bufp += MAP_COL_NUM ) {
		for ( j = 0; j < MAP_COL_NUM; j ++ ) {

			if ( bufp[j] == 'Z' ) {
				map_setup_spaceship( map, spaceship, i, j );
			}
			else if ( bufp[j] == 'A' ) {
				map_setup_enemy( map, enemies, ENEMY_BALL, i, j );
			}
			else if ( bufp[j] == 'B' ) {
				map_setup_enemy( map, enemies, ENEMY_MISSILER, i, j );
			}
			else if ( bufp[j] == 'C' ) {
				map_setup_enemy( map, enemies, ENEMY_SCULL, i, j );
			}
			else if ( bufp[j] == 'D' ) {
				map_setup_enemy( map, enemies, ENEMY_FORTRESS, i, j );
			}
			else if ( bufp[j] == 'E' ) {
				map_setup_enemy( map, enemies, ENEMY_FLYER, i, j );
			}
			else if ( bufp[j] == 'K' ) {
				int image_index = map->image_index[ INDEX_IMAGE_KEY_BLONZE + level ];
				map_setup_key( map, key, image_index, i, j, level );
			}

			map->data[i][j]=bufp[j];
		}
	}

	map->level = level;
	return MAP_AVAILABLE;
}


static void
map_setup_spaceship( Map* map, Entity* spaceship, int row, int col )
{
	map->center_x =	MAP_BLOCK_WIDTH  * col + SPACESHIP_WIDTH  / 2;
	map->center_y =	MAP_BLOCK_HEIGHT * row + SPACESHIP_HEIGHT / 2;
	entity_set_position( spaceship, map->center_x, map->center_y );
}


static void
map_setup_enemy( Map* map, RLlist* enemies, EnemyType type, int row, int col )
{
	Entity enemy;
	double x;
	double y;

	enemy_init( &enemy, map->gt_app, map->sound, type, &( map->screen ) );

	x = MAP_BLOCK_WIDTH  * col + entity_get_width( &enemy )  / 2;
	y = MAP_BLOCK_HEIGHT * row + entity_get_height( &enemy ) / 2;

	entity_set_position( &enemy, x, y );
	rlib_list_insert( enemies, -1, ( unsigned char* )( &enemy ), sizeof( Entity ) );
}


static void
map_setup_key( Map* map, Entity* key, int index, int row, int col, int level )
{
	double x;
	double y;

	item_init( key, map->gt_app, ITEM_KEY, &( map->screen ), index );

	x = MAP_BLOCK_WIDTH  * col + entity_get_width( key )  / 2;
	y = MAP_BLOCK_HEIGHT * row + entity_get_height( key ) / 2;

	entity_set_position( key, x, y );
}


/*!
 * @brief Gets the current map level.
 * @retval -1 when the current level is not available or map is NULL.
 *         Otherwise, returns the current map level.
 */
int
map_get_current_level( Map* map )
{
	if ( map == NULL ) {
		return -1;
	}

	return map->level;
}


/*!
 * Checks if the specified rectangle ( represents a moving entity ) 
 * hits some of map objects. Specify current velocity of rectangle 
 * to vx and vy. When this check ends, re-computed velocity value 
 * will be saved to vx and vy. For example, if the specified rectangle 
 * cannot move because there is a wall or something, rectangle might 
 * change its direction because of the reflection. Use these values 
 * as the new velocity of the specified rectangle.
 *
 * @param map  [in]  Map data structure.
 * @param rect [in]  Rectangle to do this check.
 * @param vx   [i/o] Specify x element of the rectangle's velocity.
 * @param vy   [i/o] Specify y element of the rectangle's velocity.
 * @param type [out] Type of the block that hits.
 *
 * @retval 1 when the specified rectangle can move. Else returns 0.
 */
int
map_is_movable( Map* map, RLrect* rect, double* vx, double* vy, unsigned char* type )
{
	int min_row;
	int max_row;
	int min_col;
	int max_col;
	int i;
	int j;

	if ( map == NULL || rect == NULL || vx == NULL || vy == NULL ) {
		return 0;
	}

	map_get_region_for_collision_check( 
		rect, vx, vy, &min_row, &max_row, &min_col, &max_col );

	for ( i = min_row; i <= max_row; i ++ ) {
		for ( j = min_col; j <= max_col; j ++ ) {
			if ( map_is_block( map->data[i][j] ) ) {
				RLrect block;
				int lt_x;
				int lt_y;

				if ( map_is_key_lock( map->data[i][j] ) && ! map->show_key_lock ) {
					continue;
				}

				lt_x = MAP_BLOCK_WIDTH  * j;
				lt_y = MAP_BLOCK_HEIGHT * i;

				rlib_rect_init( &block, lt_x, lt_y, MAP_BLOCK_WIDTH, MAP_BLOCK_HEIGHT );

				if ( rlib_rect_overlap( rect, &block ) ) {
					map_recompute_velocity( map, rect, i, j, vx, vy );
					*type = map->data[i][j];
					return 0;
				}
			}
		}
	}

	return 1;
}


static void
map_get_region_for_collision_check( RLrect* rect, double* vx, double* vy,
	    int* min_row, int* max_row, int* min_col, int* max_col )
{
	*min_col = ( rect->lefttop_x / MAP_BLOCK_WIDTH );
	*min_row = ( rect->lefttop_y / MAP_BLOCK_HEIGHT );
	*max_col = ( rect->lefttop_x + rect->width )  / MAP_BLOCK_WIDTH;
	*max_row = ( rect->lefttop_y + rect->height ) / MAP_BLOCK_HEIGHT;

	*min_col = ( *min_col <= 0 ) ? 0 : *min_col - 1;
	*min_row = ( *min_row <= 0 ) ? 0 : *min_row - 1;
	*max_col = ( *max_col >= MAP_COL_NUM - 1 ) ? MAP_COL_NUM - 1 : *max_col + 1;
	*max_row = ( *max_row >= MAP_ROW_NUM - 1 ) ? MAP_ROW_NUM - 1 : *max_row + 1;
}


static void
map_recompute_velocity( 
	Map* map, RLrect* rect, int block_row, int block_col, double* vx, double* vy )
{
	int row = ( rect->lefttop_y + rect->height / 2 ) / MAP_BLOCK_WIDTH;
	int col = ( rect->lefttop_x + rect->width  / 2 ) / MAP_BLOCK_HEIGHT;

	if ( row == block_row ) {
		( *vx ) *= -1.0;
		return;
	}

	if ( col == block_col ) {
		( *vy ) *= -1.0;
		return;
	}

	if ( ( row < block_row && ( *vy ) < 0 ) || ( row > block_row && ( *vy ) > 0 ) ) {
		if ( ( col < block_col && ( *vx ) > 0 )|| ( col > block_col && ( *vx ) < 0 ) ) {
			( *vx ) *= -1.0;
		}

		return;
	}

	if ( ( col < block_col && ( *vx ) < 0 ) || ( col > block_col && ( *vx ) > 0 ) ) {
		( *vy ) *= -1.0;
		return;
	}

	if ( fabs( *vx ) < fabs( *vy ) ) {
		if ( ! map_is_block( map->data[ block_row ][ col ] ) ) {
			( *vx ) *= -1.0;
		}
		else {
			( *vy ) *= -1.0;
		}
	}
	else {
		if ( ! map_is_block( map->data[ row ][ block_col ] ) ) {
			( *vy ) *= -1.0;
		}
		else {
			( *vx ) *= -1.0;
		}
	}

	return;
}


static int
map_is_block( unsigned char map_element )
{
	if ( map_element == MAP_NORMAL_BLOCK  || 
	     map_element == MAP_ENTRANCE_LOCK || 
	     map_element == MAP_KEY_LOCK ) {

		return 1;
	}

	return 0;
}


static int
map_is_normal_block( unsigned char map_element )
{
	if ( map_element == MAP_NORMAL_BLOCK ) {
		return 1;
	}

	return 0;
}


/**
 * Checks if the specified element is entrance lock block or not.
 * Returns 1 when it is an entrance lock block, 0 if not.
 */
int
map_is_entrance_lock( unsigned char map_element )
{
	if ( map_element == MAP_ENTRANCE_LOCK ) {
		return 1;
	}

	return 0;
}


static int
map_is_key_lock( unsigned char map_element )
{
	if ( map_element == MAP_KEY_LOCK ) {
		return 1;
	}

	return 0;
}


/*!
 * Specify 0 when you don't want to draw radar center point.
 * In contrast, specify !0 when you want to draw it.
 */
void
map_show_radar_center( Map* map, int flag )
{
	if ( map == NULL ) {
		return;
	}

	map->show_radar_center = flag;
}


/*!
 * Specify 0 when you don't want to draw entrance lock.
 * In contrast, specify !0 when you want to draw it.
 */
void
map_show_entrance_lock( Map* map, int flag )
{
	if ( map == NULL ) {
		return;
	}

	map->show_entrance_lock = flag;
}


/*!
 * Specify 0 when you don't want to draw key lock.
 * In contrast, specify !0 when you want to draw it.
 */
void
map_show_key_lock( Map* map, int flag )
{
	if ( map == NULL ) {
		return;
	}

	map->show_key_lock = flag;
}


/* End of map.c */
