/*
 * Copyright (c) 2005,2006 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief Handles an information panel that will be located 
 *        on the right side of the screen while you're in play.
 *
 * "$Id: playinfo.c,v 1.1.1.1 2006/07/22 18:07:56 ryo Exp $"
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <stdio.h>
#include <string.h>

#ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
#endif

#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif

#ifdef HAVE_GETTIMEOFDAY
#  define GETTIME(time) gettimeofday( (time), NULL )
#endif

#include "RLgt.h"
#include "RLrect.h"
#include "RLtimer.h"
#include "playinfo.h"
#include "resource.h"


#define PLAYINFO_MAX_SEC        300
#define PLAYINFO_MAX_ENERGY     300
#define PLAYINFO_ENERGY_GREEN    70
#define PLAYINFO_ENERGY_CAUTION  30

#define PLAYINFO_DIGIT_WIDTH     26
#define PLAYINFO_DIGIT_HEIGHT    50

#define PLAYINFO_KEY_WIDTH       40
#define PLAYINFO_KEY_HEIGHT      40


static void playinfo_get_left_time( PlayInfo* playinfo, int* min, int* sec );
static void playinfo_draw_title( PlayInfo* playinfo );
static void playinfo_draw_radar( PlayInfo* playinfo );
static void playinfo_draw_item( PlayInfo* playinfo );
static void playinfo_draw_energy( PlayInfo* playinfo );
static void playinfo_draw_timer( PlayInfo* playinfo );


/*!
 * Initializes information panel data structure.
 *
 * @param playinfo    [i/o] The information panel data storage.
 * @param app         [in]  GUI toolkit context.
 * @param screen      [in]  Screen definition on which the information panel should be drawn.
 * @param image_index [in]  Image indexes that will be used to draw the information panel.
 */
void
playinfo_init( PlayInfo* playinfo, RLgtApp* app, RLrect* screen, int* image_index )
{
	if ( playinfo == NULL || app == NULL || screen == NULL ) {
		return;
	}

	memset( playinfo, 0, sizeof( PlayInfo ) );
	GETTIME( &( playinfo->previous_time ) );

	playinfo->gt_app           = app;
	playinfo->screen.lefttop_x = screen->lefttop_x;
	playinfo->screen.lefttop_y = screen->lefttop_y;
	playinfo->screen.width     = screen->width;
	playinfo->screen.height    = screen->height;
	playinfo->image_index      = image_index;
}


/*!
 * Gets current time and calculates how long time has elapsed
 * since the play of the current map level started.
 */
void
playinfo_update( PlayInfo* playinfo )
{
#ifdef HAVE_SYS_TIME_H
	struct timeval current_time;
	struct timeval diff_time;
	struct timeval elapsed_time;

	if ( playinfo == NULL ) {
		return;
	}

	GETTIME( &current_time );

	RLIB_TIMER_DIFF( &diff_time, &current_time, &( playinfo->previous_time ) );
	RLIB_TIMER_COPY( &elapsed_time, &( playinfo->elapsed_time ) );
	RLIB_TIMER_ADD( &( playinfo->elapsed_time ), &elapsed_time, &diff_time );
	RLIB_TIMER_COPY( &( playinfo->previous_time ), &current_time );
#endif
}


#define MARGIN 5


/*!
 * Draws the informatin panel onto the screen.
 */
void
playinfo_draw( PlayInfo* playinfo )
{
	if ( playinfo == NULL || playinfo->gt_app == NULL ) {
		return;
	}

	/* Fills all region. */
	rlib_gt_set_foreground_color( 
		playinfo->gt_app,
		( unsigned int )( RL_GT_MAX_RGB * 0.0 ),
		( unsigned int )( RL_GT_MAX_RGB * 0.2 ),
		( unsigned int )( RL_GT_MAX_RGB * 0.4 ) );

	rlib_gt_fill_rectangle( 
		playinfo->gt_app,
		playinfo->screen.lefttop_x, 
		playinfo->screen.lefttop_y,
		playinfo->screen.width,
		playinfo->screen.height );

	/* Left edge. */
	rlib_gt_set_foreground_color( 
		playinfo->gt_app,
		( unsigned int )( RL_GT_MAX_RGB * 0.1 ),
		( unsigned int )( RL_GT_MAX_RGB * 0.3 ),
		( unsigned int )( RL_GT_MAX_RGB * 0.6 ) );

	rlib_gt_fill_rectangle( 
		playinfo->gt_app,
		playinfo->screen.lefttop_x,
		playinfo->screen.lefttop_y,
		2,
		playinfo->screen.height );

	/* Top edge. */
	rlib_gt_fill_rectangle( 
		playinfo->gt_app,
		playinfo->screen.lefttop_x,
		playinfo->screen.lefttop_y,
		playinfo->screen.width,
		2 );

	/* Bottom edge. */
	rlib_gt_set_foreground_color( 
		playinfo->gt_app,
		( unsigned int )( RL_GT_MAX_RGB * 0.0 ),
		( unsigned int )( RL_GT_MAX_RGB * 0.1 ),
		( unsigned int )( RL_GT_MAX_RGB * 0.3 ) );

	rlib_gt_fill_rectangle( 
		playinfo->gt_app,
		playinfo->screen.lefttop_x,
		playinfo->screen.lefttop_y + playinfo->screen.height - 2,
		playinfo->screen.width,
		2 );

	/* Right edge. */
	rlib_gt_fill_rectangle( 
		playinfo->gt_app,
		playinfo->screen.lefttop_x + playinfo->screen.width  - 2,
		playinfo->screen.lefttop_y,
		2,
		playinfo->screen.height );

	playinfo_draw_title( playinfo );
	playinfo_draw_radar( playinfo );
	playinfo_draw_energy( playinfo );
	playinfo_draw_item( playinfo );
	playinfo_draw_timer( playinfo );
}


static void
playinfo_draw_title( PlayInfo* playinfo )
{
	int image_w;
	int image_h;
	int index = playinfo->image_index[ INDEX_IMAGE_TITLE_INFO ];
	int lt_x  = playinfo->screen.lefttop_x;
	int lt_y  = playinfo->screen.lefttop_y;

	rlib_gt_get_image_size( playinfo->gt_app, index, &image_w, &image_h );
	rlib_gt_draw_image( playinfo->gt_app, index, 0, 0, image_w, image_h, lt_x, lt_y );
}


static void
playinfo_draw_radar( PlayInfo* playinfo )
{
	int lt_x   = playinfo->screen.lefttop_x  + MARGIN;
	int lt_y   = playinfo->screen.lefttop_y  + playinfo->screen.height / 4 + MARGIN;
	int width  = playinfo->screen.width      - MARGIN * 2;
	int height = playinfo->screen.height / 4 - MARGIN * 2;

	rlib_gt_set_foreground_color( playinfo->gt_app, 0, 0, 0 );
	rlib_gt_fill_rectangle( playinfo->gt_app, lt_x, lt_y, width, height );
}


/*!
 * Gets a rectangle represents radar screen.
 * This function does nothing when NULL is specified as args.
 */
void
playinfo_get_radar_rect( PlayInfo* playinfo, RLrect* rect )
{
	if ( playinfo == NULL || rect == NULL ) {
		return;
	}

	rect->lefttop_x = playinfo->screen.lefttop_x  + MARGIN;
	rect->lefttop_y = playinfo->screen.lefttop_y  + playinfo->screen.height / 4 + MARGIN;
	rect->width     = playinfo->screen.width      - MARGIN * 2;
	rect->height    = playinfo->screen.height / 4 - MARGIN * 2;
}


static void
playinfo_draw_energy( PlayInfo* playinfo )
{
	int lt_x   = playinfo->screen.lefttop_x + MARGIN;
	int lt_y   = playinfo->screen.lefttop_y + playinfo->screen.height / 2 + MARGIN * 3;
	int width  = playinfo->screen.width  - MARGIN * 2;
	int height = playinfo->screen.height / 8 - MARGIN * 6;
	int e_width;

	rlib_gt_set_foreground_color( playinfo->gt_app, 0, 0, 0 );
	rlib_gt_fill_rectangle( playinfo->gt_app, lt_x, lt_y, width, height );

	if ( playinfo->energy == 0 ) {
		return;
	}

	if ( playinfo->energy > PLAYINFO_ENERGY_GREEN ) {
		rlib_gt_set_foreground_color( playinfo->gt_app, 0, RL_GT_MAX_RGB, 0 );
	}
	else if ( playinfo->energy > PLAYINFO_ENERGY_CAUTION ) {
		rlib_gt_set_foreground_color( playinfo->gt_app, RL_GT_MAX_RGB, RL_GT_MAX_RGB, 0 );
	}
	else {
		rlib_gt_set_foreground_color( playinfo->gt_app, RL_GT_MAX_RGB, 0, 0 );
	}

	e_width = width * playinfo->energy / PLAYINFO_MAX_ENERGY;
	rlib_gt_fill_rectangle( playinfo->gt_app, lt_x, lt_y, e_width, height );
}


static void
playinfo_draw_item( PlayInfo* playinfo )
{
	int lt_x   = playinfo->screen.lefttop_x + MARGIN;
	int lt_y   = playinfo->screen.lefttop_y + playinfo->screen.height * 5 / 8 + MARGIN;
	int width  = playinfo->screen.width  - MARGIN * 2;
	int height = playinfo->screen.height / 8 - MARGIN * 2;

	int found_key = 0;
	int i;

	rlib_gt_set_foreground_color( playinfo->gt_app, 0, 0, 0 );
	rlib_gt_fill_rectangle( playinfo->gt_app, lt_x, lt_y, width, height );

	for ( i = 0; i < PLAYINFO_KEY_NUM; i ++ ) {
		if ( playinfo->keys[i] ) {
			rlib_gt_draw_image(
				playinfo->gt_app,
				playinfo->image_index[ INDEX_IMAGE_KEY_BLONZE + i ],
				0,
				0,
				PLAYINFO_KEY_WIDTH,
				PLAYINFO_KEY_HEIGHT,
				lt_x,
				lt_y + MARGIN );

			lt_x += ( PLAYINFO_KEY_WIDTH + MARGIN );
			found_key = 1;
		}
	}

	if ( ! found_key ) {
		rlib_gt_draw_image(
			playinfo->gt_app,
			playinfo->image_index[ INDEX_IMAGE_NOITEM ],
			0,
			0,
			width,
			height,
			lt_x,
			lt_y );
	}
}


static void
playinfo_draw_timer( PlayInfo* playinfo )
{
	int image_w;
	int image_h;
	int index = playinfo->image_index[ INDEX_IMAGE_TIME3 ];
	int lt_x  = playinfo->screen.lefttop_x;
	int lt_y  = playinfo->screen.lefttop_y + playinfo->screen.height * 3 / 4;

#define DIGIT_NUM 5
#define OFFSET_DM 10

	int digit[ DIGIT_NUM ];
	int min;
	int sec;
	int i;

	rlib_gt_get_image_size( playinfo->gt_app, index, &image_w, &image_h );
	rlib_gt_draw_image( playinfo->gt_app, index, 0, 0, image_w, image_h, lt_x, lt_y );
	lt_y += playinfo->screen.height / 8;
/*	playinfo_get_elapsed_time( playinfo, &min, &sec ); */
	playinfo_get_left_time( playinfo, &min, &sec );

	digit[0] = min / 10;
	digit[1] = min - digit[0] * 10;
	digit[2] = ':';
	digit[3] = sec / 10;
	digit[4] = sec - digit[3] * 10;

	for ( i = 0; i < DIGIT_NUM; i ++ ) {
		if ( digit[i] == ':' ) {
			index = OFFSET_DM;
		}
		else {
			index = digit[i];
		}

		rlib_gt_draw_image(
			playinfo->gt_app,
			playinfo->image_index[ INDEX_IMAGE_NUMBERS0_0 + index ],
			0,
			0,
			PLAYINFO_DIGIT_WIDTH,
			PLAYINFO_DIGIT_HEIGHT,
			lt_x + MARGIN,
			lt_y );

		lt_x += PLAYINFO_DIGIT_WIDTH;
	}
}


/*!
 * Gets elapsed time. If elapsed time exceeds 60 minites,
 * min 59 and sec 59 will be returned.
 */
void
playinfo_get_elapsed_time( PlayInfo* playinfo, int* min, int* sec )
{
#ifdef HAVE_SYS_TIME_H
	*min = playinfo->elapsed_time.tv_sec / 60;
	*sec = playinfo->elapsed_time.tv_sec % 60;
#endif
	if ( *min >= 60 ) {
		*min = 59;
		*sec = 59;
	}
}


static void
playinfo_get_left_time( PlayInfo* playinfo, int* min, int* sec )
{
	int left_sec;

#ifdef HAVE_SYS_TIME_H
	left_sec = ( playinfo->elapsed_time.tv_sec >= PLAYINFO_MAX_SEC ) ?
		0 : PLAYINFO_MAX_SEC - playinfo->elapsed_time.tv_sec;
#endif	

	*min = left_sec / 60;
	*sec = left_sec % 60;
}


/*!
 * Checks if time is left.
 * @retval 0 when time is not left. 
 */
int
playinfo_is_time_left( PlayInfo* playinfo )
{
	if ( playinfo->elapsed_time.tv_sec >= PLAYINFO_MAX_SEC ) {
		return 0;
	}

	return 1;
}


/*!
 * Resets previous playing time to calculate elapsed time.
 */
void
playinfo_reset_previous_time( PlayInfo* playinfo )
{
	GETTIME( &( playinfo->previous_time ) );
}


/*!
 * Sets the energy amount. amount should be within 0 - PLAYINFO_MAX_ENERGY.
 */
void
playinfo_set_energy( PlayInfo* playinfo, int amount )
{
	if ( playinfo == NULL || amount < 0 || amount > PLAYINFO_MAX_ENERGY ) {
		return;
	}

	playinfo->energy = amount;
}


/*!
 * Sets the key status with the specified number to "found".
 * number should be within 0 - PLAYINFO_KEY_NUM.
 */
void
playinfo_found_key( PlayInfo* playinfo, int number )
{
	if ( playinfo == NULL || number < 0 || number >= PLAYINFO_KEY_NUM ) {
		return;
	}

	playinfo->keys[ number ] = 1;
}


/*!
 * Checks the key status with the specified number.
 * number should be within 0 - PLAYINFO_KEY_NUM.
 * Returns 1 when specified key status is "found", else returns 0.
 */
int
playinfo_is_key_found( PlayInfo* playinfo, int number )
{
	if ( playinfo == NULL || number < 0 || number >= PLAYINFO_KEY_NUM ) {
		return 0;
	}

	return playinfo->keys[ number ];
}


/* End of playinfo.c */
