/*
 * Copyright (c) 2005,2006 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief Main header file of "Space Maze".
 *
 * "$Id: spacemaze.h,v 1.4 2006/10/09 09:10:47 ryo Exp $"
 */

#ifndef ___SPACEMAZE_H_INCLUDED_
#define ___SPACEMAZE_H_INCLUDED_


#include <time.h>

#include "RLgt.h"
#include "RLlist.h"
#include "RLsound.h"
#include "map.h"
#include "playinfo.h"
#include "spaceship.h"
#include "star.h"


/*
 * Maximum fps ( frames per second ).
 */
#define SPACEMAZE_MAX_FPS            60

/*
 * Minimum fps.
 */
#define SPACEMAZE_MIN_FPS            30

#define SPACEMAZE_DEFAULT_FPS        45
#define SPACEMAZE_SCREEN_WIDTH       640
#define SPACEMAZE_SCREEN_HEIGHT      480

#define SPACEMAZE_SCREEN_PLAY_WIDTH  500
#define SPACEMAZE_SCREEN_PLAY_HEIGHT 480
#define SPACEMAZE_SCREEN_INFO_WIDTH  ( SPACEMAZE_SCREEN_WIDTH  - SPACEMAZE_SCREEN_PLAY_WIDTH )
#define SPACEMAZE_SCREEN_INFO_HEIGHT SPACEMAZE_SCREEN_HEIGHT

#define SPACEMAZE_STAR_COUNT 60


/* 
 * Game status. 
 * These values will be used as an index of array within
 * game_manager.c
 */
#define STATE_TITLE     0
#define STATE_SELECT    1
#define STATE_BOSS      2
#define STATE_PLAYING   3
#define STATE_PAUSE     4
#define STATE_CLEAR     5
#define STATE_GAMEOVER  6
#define STATE_ENDING    7
#define STATE_INVALID   8


/*!
 * @brief Records.
 */
typedef struct
{
	int keys[ MAP_MAX_LEVEL + 1 ];
	int mins[ MAP_MAX_LEVEL + 1 ];
	int secs[ MAP_MAX_LEVEL + 1 ];
}
Record;


/*!
 * @brief Title screen status.
 */
typedef struct
{
	int initialized;
	int is_start_focused;
	int is_load_focused;
	int is_exit_focused;
	int is_selected;
	int effect_selected_frame_count;
	int effect_loadfail_frame_count;
}
TitleScreen;


/*!
 * @brief Select screen status.
 */
typedef struct
{
	int initialized;
	int is_maplevel_focused[ MAP_MAX_LEVEL + 1 ];
	int is_back_focused;
	int is_save_focused;
	int is_selected;
	int previous_level;
	int effect_selected_frame_count;
	int effect_savedone_frame_count;
	int effect_savefail_frame_count;
}
SelectScreen;


/*!
 * @brief Playing screen status.
 */
typedef struct
{
	int effect_start_frame_count;
	int effect_evacuate_frame_count;
	int decrease_energy_frame_count;
	int has_key;
}
PlayScreen;


#include "resource.h"


/*!
 * @brief "Space Maze" application context.
 */
typedef struct
{
	RLgtApp        gt;
	RLsoundContext sound;

	int            image_index[ MAX_IMAGE_COUNT ];
	int            wave_index[ MAX_WAVE_COUNT ];
	int            is_debug_on;
	int            initial_number_of_enemies;
	int            effect_blink_frame_count;
	int            game_state;
	int            next_state;

	/* Objects. */
	Map            map;
	PlayInfo       playinfo;
	Record         record;
	Entity         key;
	Entity         spaceship;
	RLlist         enemies;
	RLlist         s_missiles;
	RLlist         e_missiles;
	RLlist         stars;

	/* Screen related variables. */
	int            mode_key;
	TitleScreen    title_screen;
	SelectScreen   select_screen;
	PlayScreen     play_screen;
}
SpaceMazeApp;


#define INIT_STARS( app ) { \
        Entity star; \
        int i; \
\
	rlib_list_init( &( (app)->stars ) ); \
\
        for ( i = 0; i < SPACEMAZE_STAR_COUNT; i ++ ) { \
                star_init( &star, &( (app)->gt ) ); \
                rlib_list_insert( &( (app)->stars ), -1, ( unsigned char* )( &star ), sizeof( Entity ) ); \
        }\
}


#define UPDATE_STARS( app ) { \
        int i; \
\
        for ( i = 0; i < rlib_list_size( &( (app)->stars ) ); i ++ ) { \
                RLlistData* list_item = rlib_list_get( &( (app)->stars ), i ); \
                Entity* entity = ( Entity* )( list_item->buf ); \
                entity_update( entity, NULL ); \
        }\
}


#define DRAW_STARS( app ) { \
        int i; \
\
        for ( i = 0; i < rlib_list_size( &( (app)->stars ) ); i ++ ) { \
                RLlistData* list_item = rlib_list_get( &( (app)->stars ), i ); \
                Entity* entity = ( Entity* )( list_item->buf ); \
                entity_draw( entity ); \
        }\
}


#define UPDATE_RECORD_TIME( app, level, min, sec ) { \
        (app)->record.mins[(level)] = (min); \
        (app)->record.secs[(level)] = (sec); \
}


#define SRAND() ( srand( ( unsigned int )time( NULL ) ) )

/* For debug use. */
#define DUMP_CONTEXT( app ) { \
        fprintf( stderr, "==== Some member of application context ===\n" );\
        fprintf( stderr, "is_debug_on               = %d\n", (app)->is_debug_on );\
        fprintf( stderr, "initial_number_of_enemies = %d\n", (app)->initial_number_of_enemies );\
        fprintf( stderr, "effect_blink_frame_count  = %d\n", (app)->effect_blink_frame_count );\
        fprintf( stderr, "game_state                = %d\n", (app)->game_state );\
        fprintf( stderr, "next_state                = %d\n", (app)->next_state );\
        fprintf( stderr, "mode_key                  = %d\n", (app)->mode_key );\
}


#endif /* ___SPACEMAZE_H_INCLUDED_ */

/* End of spacemaze.h */
