/*
 * Copyright (c) 2006 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief Handles in-play screen ( the last stage ).
 *
 * "$Id: state_boss.c,v 1.1.1.1 2006/07/22 18:07:56 ryo Exp $"
 */

#include "RLgt.h"
#include "RLlist.h"
#include "RLrect.h"
#include "bg.h"
#include "map.h"
#include "playinfo.h"
#include "spacemaze.h"
#include "spaceship.h"
#include "state.h"


#define Y_CENTER_DESTROY 200


/*!
 * Initializes in-play related variables of application context.
 * @param app   [i/o] Application context.
 * @param level [in]  Map level to load.
 */
void
state_boss_init( SpaceMazeApp* app, int level )
{
	state_playing_init( app, level );
}


/*!
 * This function will be called within main loop.
 *
 * - Checks keyboard event ( These will be done within rlib_gt_process_event() ).
 *   - If escape key is pressed and space ship is not exploding,
 *     changes game state to STATE_SELECT ( state_playing_key_pressed() ).
 *
 *   - If 'p' key is pressed, changes game state to STATE_PAUSE
 *     ( state_playing_key_pressed() ).
 *
 *   - Keyboard event checks for the space ship 
 *     ( spaceship_key_pressed(), spaceship_key_released() ).
 *
 * - If the space ship is dead, changes game state to STATE_GAMEOVER.
 * - If all enemies are dead, changes game state to STATE_CLEAR.
 * - Updates star object's information.
 * - Checks collision between the space ship and map objects.
 * - Checks collision between enemies, missiles, map objects.
 *   And also checks collision between missiles and the space ship.
 * - Updates all object's information.
 *
 * @param app [i/o] Application context.
 * @retval Next game state. 
 */
int
state_boss_update( SpaceMazeApp* app )
{
	RLrect s_rect;
	double vx;
	double vy;

	if ( app == NULL ) {
		return STATE_INVALID;
	}

	if ( rlib_gt_process_event( &( app->gt ) ) != 0 ) {
		return app->next_state;
	}

	if ( ! entity_is_alive( &( app->spaceship ) ) && ! entity_is_exploding( &( app->spaceship ) ) ) {
		map_show_radar_center( &( app->map ), 0 );
		app->effect_blink_frame_count = 0;
		state_gameover_set_event_handler( app );
		return STATE_GAMEOVER;
	}

	if ( rlib_list_size( &( app->enemies ) ) == 0 ) {
		state_clear_init( app );
		return STATE_CLEAR;
	}

	if ( app->play_screen.effect_start_frame_count > 0 ) {
		app->play_screen.effect_start_frame_count --;
	}

	if ( ! playinfo_is_time_left( &( app->playinfo ) ) ) {
		if ( app->play_screen.decrease_energy_frame_count <= 0 ) {
			app->play_screen.decrease_energy_frame_count = state_get_framecount_decrease_energy();
			entity_decrease_energy( &( app->spaceship ), STATE_DECREASE_ENERGY_AMOUNT );
		}

		app->play_screen.decrease_energy_frame_count --;
	}

	UPDATE_STARS( app );

	if ( entity_check( &( app->spaceship ), &s_rect, &vx, &vy ) ) {
		unsigned char type;
		if ( ! map_is_movable( &( app->map ), &s_rect, &vx, &vy, &type ) ) {
			entity_set_velocity( &( app->spaceship ), vx, vy );
		}
	}

	state_update_enemies( app, &s_rect );
	state_update_enemy_missiles( app, &s_rect );
	state_update_ship_missiles( app );
	state_update_items( app, &s_rect );
	entity_update( &( app->spaceship ), NULL );
	map_update( &( app->map ), &( app->spaceship ) );
	playinfo_set_energy( &( app->playinfo ), entity_get_energy( &( app->spaceship ) ) );
	playinfo_update( &( app->playinfo ) );

	return app->game_state;
}


/*!
 * Draws in-play screen.
 * @param app [i/o] Application context.
 */
void
state_boss_draw( SpaceMazeApp* app )
{
	if ( app == NULL ) {
		return;
	}

	bg_draw( &( app->gt ) );
	DRAW_STARS( app );
	state_draw_items( app );
	state_draw_enemy_missiles( app );
	state_draw_enemies( app );
	state_draw_spaceship_missiles( app );
	entity_draw( &( app->spaceship ) );
	map_draw( &( app->map ) );
	playinfo_draw( &( app->playinfo ) );
	map_draw_radar( &( app->map ) );
	state_draw_enemy_info( app );

	if ( app->play_screen.effect_start_frame_count > 0 ) {
		DRAW_IMAGE( app, INDEX_IMAGE_DESTROY, 
			SPACEMAZE_SCREEN_PLAY_WIDTH/ 2, Y_CENTER_DESTROY );
	}
}


/* End of state_boss.c */
