/*
 * Copyright (c) 2005,2006 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief Handles map-clear screen.
 *
 * "$Id: state_clear.c,v 1.2 2006/07/24 06:00:41 ryo Exp $"
 */

#include "RLgt.h"
#include "RLsound.h"
#include "bg.h"
#include "entity.h"
#include "map.h"
#include "playinfo.h"
#include "spacemaze.h"
#include "state.h"


#define X_CENTER_LEVEL    130
#define X_CENTER_DIGIT    220
#define X_CENTER_CLEAR    370
#define Y_CENTER_CLEAR    200
#define Y_CENTER_CONTINUE 260

#define DIGIT_WIDTH  26
#define DIGIT_HEIGHT 50


static int state_clear_key_pressed( void* data );


/*!
 * Initializes clear screen related variables of application context.
 * @param app [i/o] Application context.
 */
void
state_clear_init( SpaceMazeApp* app )
{
	rlib_sound_stop_all( &( app->sound ) );
	rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_CLEAR ], 0 );
	state_clear_set_event_handler( app );
}


/*!
 * Sets event handler.
 */
void
state_clear_set_event_handler( SpaceMazeApp* app )
{
	if ( app != NULL ) {
		rlib_gt_clear_event_handler( &( app->gt ) );
		rlib_gt_add_event_handler( &( app->gt ),
			RL_GT_KEY_PRESSED, state_clear_key_pressed, ( void* )app );
	}
}


/*!
 * This function will be called within main loop.
 *
 * - Checks keyboard event ( These will be done within rlib_gt_process_event() ).
 *   - If escape key is pressed, changes game state to STATE_ENDING 
 *     if the current map level is the last one, in other cases changes 
 *     game state to STATE_SELECT.
 * - Updates star object's information.
 *
 * @param app [i/o] Application context.
 * @retval Next game state. 
 */
int
state_clear_update( SpaceMazeApp* app )
{
	if ( app == NULL ) {
		return STATE_INVALID;
	}

	if ( rlib_gt_process_event( &( app->gt ) ) != 0 ) {
		return app->next_state;
	}

	app->effect_blink_frame_count ++;
	UPDATE_STARS( app );
	return app->game_state;
}


/*!
 * Draws map-clear screen.
 * @param app [in] Application context.
 */
void
state_clear_draw( SpaceMazeApp* app )
{
	if ( app == NULL ) {
		return;
	}

	bg_draw( &( app->gt ) );
	DRAW_STARS( app );
	state_draw_items( app );
	state_draw_enemy_missiles( app );
	state_draw_enemies( app );
	state_draw_spaceship_missiles( app );
	entity_draw( &( app->spaceship ) );
	map_draw( &( app->map ) );
	playinfo_draw( &( app->playinfo ) );
	map_draw_radar( &( app->map ) );

	if ( map_get_current_level( &( app->map ) ) == MAP_MAX_LEVEL ) {
		DRAW_IMAGE( app, INDEX_IMAGE_SAVE_GALAXY,
		    SPACEMAZE_SCREEN_PLAY_WIDTH / 2, Y_CENTER_CLEAR );
	}
	else {
		DRAW_IMAGE( app, INDEX_IMAGE_LEVEL0, X_CENTER_LEVEL, Y_CENTER_CLEAR );

		rlib_gt_draw_image(
			&( app->gt ),
			INDEX_IMAGE_NUMBERS0_0 + ( map_get_current_level( &( app->map ) ) + 1 ),
			0,
			0,
			DIGIT_WIDTH,
			DIGIT_HEIGHT,
			X_CENTER_DIGIT,
			Y_CENTER_CLEAR - DIGIT_HEIGHT / 2
			);

		DRAW_IMAGE( app, INDEX_IMAGE_CLEAR, X_CENTER_CLEAR, Y_CENTER_CLEAR );
	}

	if ( app->effect_blink_frame_count < state_get_framecount_effect_blink() / 2 ) {
		DRAW_IMAGE( app, INDEX_IMAGE_CONTINUE, 
		    SPACEMAZE_SCREEN_PLAY_WIDTH / 2, Y_CENTER_CONTINUE );
	}
	else if ( app->effect_blink_frame_count >= state_get_framecount_effect_blink() ) {
		app->effect_blink_frame_count = 0;
	}
}


static int
state_clear_key_pressed( void* data )
{
	SpaceMazeApp* app = ( SpaceMazeApp* )data;

	if ( rlib_gt_key_pressed( &( app->gt ), RL_GT_KEY_ESC ) ) {
		int level;
		int min;
		int sec;

		playinfo_get_elapsed_time( &( app->playinfo ), &min, &sec );
		level = map_get_current_level( &( app->map ) );

		if ( app->record.mins[ level ] == 0 && app->record.secs[ level ] == 0 ) {
			UPDATE_RECORD_TIME( app, level, min, sec );
		}
		else if ( min < app->record.mins[ level ] ||
			  ( min == app->record.mins[ level ] && sec < app->record.mins[ level ] ) ) {

			UPDATE_RECORD_TIME( app, level, min, sec );
		}

		rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_ESC ], 0 );

		if ( map_get_current_level( &( app->map ) ) == MAP_MAX_LEVEL ) {
			state_ending_init( app );
			app->next_state = STATE_ENDING;
			return 1;
		}

		state_select_init( app );
		app->next_state = STATE_SELECT;
		return 1;
	}

	return 0;
}


/* End of state_clear.c */
