/*
 * Copyright (c) 2005,2006 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief Handles game-over screen.
 *
 * "$Id: state_gameover.c,v 1.1.1.1 2006/07/22 18:07:56 ryo Exp $"
 */

#include "RLgt.h"
#include "RLsound.h"
#include "bg.h"
#include "map.h"
#include "playinfo.h"
#include "spacemaze.h"
#include "state.h"


#define Y_CENTER_GAMEOVER 200
#define Y_CENTER_GO_BACK  250


static int state_gameover_key_pressed( void* data );


/*!
 * Sets event handler.
 */
void
state_gameover_set_event_handler( SpaceMazeApp* app )
{
	if ( app != NULL ) {
		rlib_gt_clear_event_handler( &( app->gt ) );
		rlib_gt_add_event_handler( &( app->gt ),
			RL_GT_KEY_PRESSED, state_gameover_key_pressed, ( void* )app );
	}
}


/*!
 * This function will be called within main loop.
 *
 * - Checks keyboard event ( These will be done within rlib_gt_process_event() ).
 *   - If escape key is pressed, change game state to STATE_TITLE.
 * - Updates star objects information.
 * - Checks collision between enemies, missiles, map objects. 
 *
 * @param app [i/o] Application context.
 * @retval Next game state. 
 */
int
state_gameover_update( SpaceMazeApp* app )
{
	if ( app == NULL ) {
		return STATE_INVALID;
	}

	if ( rlib_gt_process_event( &( app->gt ) ) != 0 ) {
		return app->next_state;
	}

	app->effect_blink_frame_count ++;
	UPDATE_STARS( app );
	state_update_enemies( app, NULL );
	state_update_enemy_missiles( app, NULL );
	state_update_items( app, NULL );

	return app->game_state;
}


/*!
 * Draws game-over screen.
 * @param app [in] Application context.
 */
void
state_gameover_draw( SpaceMazeApp* app )
{
	if ( app == NULL ) {
		return;
	}

	bg_draw( &( app->gt ) );
	DRAW_STARS( app );
	state_draw_items( app );
	state_draw_enemy_missiles( app );
	state_draw_enemies( app );
	state_draw_spaceship_missiles( app );
	map_draw( &( app->map ) );
	playinfo_draw( &( app->playinfo ) );
	map_draw_radar( &( app->map ) );

	DRAW_IMAGE( app, INDEX_IMAGE_GAMEOVER, 
	    SPACEMAZE_SCREEN_PLAY_WIDTH / 2, Y_CENTER_GAMEOVER  );

	if ( app->effect_blink_frame_count < state_get_framecount_effect_blink() / 2 ) {
		DRAW_IMAGE( app, INDEX_IMAGE_GO_BACK,
		    SPACEMAZE_SCREEN_PLAY_WIDTH / 2, Y_CENTER_GO_BACK );
	}
	else if ( app->effect_blink_frame_count >= state_get_framecount_effect_blink() ) {
		app->effect_blink_frame_count = 0;
	}
}


static int
state_gameover_key_pressed( void* data )
{
	SpaceMazeApp* app = ( SpaceMazeApp* )data;

	if ( rlib_gt_key_pressed( &( app->gt ), RL_GT_KEY_ESC ) ) {
		rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_ESC ], 0 );
		state_title_init( app );
		app->next_state = STATE_TITLE;
		return 1;
	}

	return 0;
}


/* End of state_gameover.c */
