/*
 * Copyright (c) 2005,2006 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief Handles in-play screen.
 *
 * "$Id: state_playing.c,v 1.3 2006/09/21 15:30:16 ryo Exp $"
 */

#include "RLgt.h"
#include "RLlist.h"
#include "RLrect.h"
#include "RLsound.h"
#include "bg.h"
#include "entity.h"
#include "map.h"
#include "playinfo.h"
#include "spacemaze.h"
#include "spaceship.h"
#include "state.h"
#include "resource.h"


#define X_CENTER_LEVEL    130
#define X_CENTER_DIGIT    220
#define X_CENTER_START    370
#define Y_CENTER_START    200
#define X_CENTER_EVACUATE 250
#define Y_CENTER_EVACUATE 200

#define DIGIT_WIDTH  26
#define DIGIT_HEIGHT 50


static int state_playing_key_pressed( void* data );


/*!
 * Initializes in-play related variables of application context.
 * @param app   [i/o] Application context.
 * @param level [in]  Map level to load.
 */
void
state_playing_init( SpaceMazeApp* app, int level )
{
	RLrect screen;
	int i;

	rlib_sound_stop_all( &( app->sound ) );
	memset( &( app->play_screen ), 0, sizeof( PlayScreen ) );

	/*
	 * Initialization of a play-info object.
	 */
	{
		screen.lefttop_x = SPACEMAZE_SCREEN_PLAY_WIDTH;
		screen.lefttop_y = 0;
		screen.width     = SPACEMAZE_SCREEN_INFO_WIDTH;
		screen.height    = SPACEMAZE_SCREEN_INFO_HEIGHT;

		playinfo_init( &( app->playinfo ), &( app->gt ), &screen, app->image_index );

		for ( i = 0; i < MAP_MAX_LEVEL; i ++ ) {
			if ( app->record.keys[i] ) {
				playinfo_found_key( &( app->playinfo ), i );
			}
		}
	}

	/*
	 * Initialization of a space ship.
	 */
	screen.lefttop_x = 0;
	screen.lefttop_y = 0;
	screen.width     = SPACEMAZE_SCREEN_PLAY_WIDTH;
	screen.height    = SPACEMAZE_SCREEN_PLAY_HEIGHT;

	spaceship_init( &( app->spaceship ), &( app->gt ), &( app->sound ), &screen );
	rlib_list_init( &( app->s_missiles ) );
	entity_set_missile_list( &( app->spaceship ), &( app->s_missiles ) );

	/*
	 * Initialization of enemy list and a map object.
	 * Then load a map data of the specified level.
	 * While map loading, each enemy data will be initialized.
	 */
	{
		RLrect radar_screen;
		playinfo_get_radar_rect( &( app->playinfo ), &radar_screen );
		map_init( &( app->map ), &( app->gt ), &( app->sound ), &screen, &radar_screen, app->image_index );
		rlib_list_init( &( app->enemies ) );
		map_load( &( app->map ), &( app->spaceship ), &( app->enemies ), &( app->key ), level );
		app->initial_number_of_enemies = rlib_list_size( &( app->enemies ) );
		map_show_radar_center( &( app->map ), 1 );
		map_show_entrance_lock( &( app->map ), 1 );
		map_show_key_lock( &( app->map ), 1 );
	}

	/*
	 * Binds missile list to the data structure of active enemies.
	 */
	rlib_list_init( &( app->e_missiles ) );

	for ( i = 0; i < rlib_list_size( &( app->enemies ) ); i ++ ) {
		RLlistData* list_item = rlib_list_get( &( app->enemies ), i );
		Entity* enemy = ( Entity* )( list_item->buf );
		entity_set_missile_list( enemy, &( app->e_missiles ) );
	}

	state_playing_set_event_handler( app );
	app->play_screen.effect_start_frame_count = state_get_framecount_effect_message();

	/* Starts sound effect after map-loading completed because it might take some time. */
	rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_START ], 0 );
}


/*!
 * Sets event handler.
 */
void
state_playing_set_event_handler( SpaceMazeApp* app )
{
	if ( app != NULL ) {
		rlib_gt_clear_event_handler( &( app->gt ) );
		rlib_gt_add_event_handler( &( app->gt ),
			RL_GT_KEY_PRESSED, state_playing_key_pressed, ( void* )app );
		spaceship_set_event_handler( &( app->spaceship ) );
	}
}

#include "RLlogger.h"

/*!
 * This function will be called within main loop.
 *
 * - Checks keyboard event ( These will be done within rlib_gt_process_event() ).
 *   - If escape key is pressed and space ship is not exploding,
 *     changes game state to STATE_SELECT ( state_playing_key_pressed() ).
 *
 *   - If 'p' key is pressed, changes game state to STATE_PAUSE
 *     ( state_playing_key_pressed() ).
 *
 *   - Keyboard event checks for the space ship
 *     ( spaceship_key_pressed(), spaceship_key_released() ).
 *
 * - If the space ship is dead, changes game state to STATE_GAMEOVER.
 * - Updates star object's information.
 * - Checks collision between the space ship and map objects.
 *   If the space ship's got a key and is on the locked blocks, 
 *   changes game state to STATE_CLEAR.
 * - Checks colission between enemies, missiles, map objects.
 *   And also checks collision between missiles and the space ship.
 * - Updates all object's information.
 *
 * @param app [i/o] Application context.
 * @retval Next game state. 
 */
int
state_playing_update( SpaceMazeApp* app )
{
	RLrect s_rect;
	double vx;
	double vy;

	if ( app == NULL ) {
		return STATE_INVALID;
	}

	if ( rlib_gt_process_event( &( app->gt ) ) != 0 ) {
		return app->next_state;
	}

	if ( ! entity_is_alive( &( app->spaceship ) ) && ! entity_is_exploding( &( app->spaceship ) ) ) {
		map_show_radar_center( &( app->map ), 0 );
		app->effect_blink_frame_count = 0;
		state_gameover_set_event_handler( app );
		return STATE_GAMEOVER;
	}

	if ( app->play_screen.effect_start_frame_count > 0 ) {
		app->play_screen.effect_start_frame_count --;
	}

	if ( app->play_screen.effect_evacuate_frame_count > 0 ) {
		app->play_screen.effect_evacuate_frame_count --;
	}

	if ( ! playinfo_is_time_left( &( app->playinfo ) ) ) {
		if ( app->play_screen.decrease_energy_frame_count <= 0 ) {
			app->play_screen.decrease_energy_frame_count = state_get_framecount_decrease_energy();
			entity_decrease_energy( &( app->spaceship ), STATE_DECREASE_ENERGY_AMOUNT );
		}

		app->play_screen.decrease_energy_frame_count --;
	}

	UPDATE_STARS( app );

	if ( entity_check( &( app->spaceship ), &s_rect, &vx, &vy ) ) {
		unsigned char type;

		if ( ! map_is_movable( &( app->map ), &s_rect, &vx, &vy, &type ) ) {
			int map_level = map_get_current_level( &( app->map ) );
			if ( map_is_entrance_lock( type ) && app->play_screen.has_key ) {
				app->record.keys[ map_level ] = 1;
				app->effect_blink_frame_count = 0;
				state_clear_init( app );
				return STATE_CLEAR;
			}

			entity_set_velocity( &( app->spaceship ), vx, vy );
		}
	}

	state_update_enemies( app, &s_rect );
	state_update_enemy_missiles( app, &s_rect );
	state_update_ship_missiles( app );
	state_update_items( app, &s_rect );
	entity_update( &( app->spaceship ), NULL );
	map_update( &( app->map ), &( app->spaceship ) );
	playinfo_set_energy( &( app->playinfo ), entity_get_energy( &( app->spaceship ) ) );
	playinfo_update( &( app->playinfo ) );

	return app->game_state;
}


/*!
 * Draws in-play screen.
 * @param app [i/o] Application context.
 */
void
state_playing_draw( SpaceMazeApp* app )
{
	if ( app == NULL ) {
		return;
	}

	bg_draw( &( app->gt ) );
	DRAW_STARS( app );
	state_draw_items( app );
	state_draw_enemy_missiles( app );
	state_draw_enemies( app );
	state_draw_spaceship_missiles( app );
	entity_draw( &( app->spaceship ) );
	map_draw( &( app->map ) );
	playinfo_draw( &( app->playinfo ) );
	map_draw_radar( &( app->map ) );
	state_draw_enemy_info( app );

	if ( app->play_screen.effect_evacuate_frame_count > 0 ) {
		DRAW_IMAGE( app, INDEX_IMAGE_EVACUATE, X_CENTER_EVACUATE, Y_CENTER_EVACUATE );
		return;
	}

	if ( app->play_screen.effect_start_frame_count > 0 ) {
		DRAW_IMAGE( app, INDEX_IMAGE_LEVEL0, X_CENTER_LEVEL, Y_CENTER_START );

		rlib_gt_draw_image(
			&( app->gt ),
			INDEX_IMAGE_NUMBERS0_0 + ( map_get_current_level( &( app->map ) ) + 1 ),
			0,
			0,
			DIGIT_WIDTH,
			DIGIT_HEIGHT,
			X_CENTER_DIGIT,
			Y_CENTER_START - DIGIT_HEIGHT / 2
			);

		DRAW_IMAGE( app, INDEX_IMAGE_LEVEL_START, X_CENTER_START, Y_CENTER_START );
	}
}


static int
state_playing_key_pressed( void* data )
{
	SpaceMazeApp* app = ( SpaceMazeApp* )data;

	if ( ! entity_is_exploding( &( app->spaceship ) ) && 
	     rlib_gt_key_pressed( &( app->gt ), RL_GT_KEY_ESC ) ) {
		rlib_sound_stop_all( &( app->sound ) );
		rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_ESC ], 0 );
		state_select_init( app );
		app->next_state = STATE_SELECT;
		return 1;
	}

	if ( rlib_gt_key_pressed( &( app->gt ), RL_GT_KEY_p ) ) {
		rlib_sound_stop_all( &( app->sound ) );
		rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_PAUSE ], 0 );
		state_pause_set_event_handler( app );
		app->next_state = STATE_PAUSE;
		return 1;
	}

	return 0;
}


/* End of state_playing.c */
