/*
 * Copyright (c) 2005,2006 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief Handles map level select screen.
 *
 * "$Id: state_select.c,v 1.2 2006/09/21 15:30:16 ryo Exp $"
 */

#include <string.h>

#include "RLgt.h"
#include "RLsound.h"
#include "bg.h"
#include "map.h"
#include "spacemaze.h"
#include "state.h"


#define FRAME_COUNT_EFFECT_SELECTED 150

#define X_CENTER_LEVEL   160
#define Y_CENTER_LEVEL   80
#define X_CENTER_DIGIT   270
#define X_CENTER_TIME    365
#define Y_MARGIN_LEVEL   100
#define X_CENTER_BACK    550
#define Y_CENTER_BACK    440
#define X_CENTER_SAVE    400
#define Y_CENTER_SAVE    Y_CENTER_BACK

#define X_CENTER_MESSAGE 320
#define Y_CENTER_MESSAGE 200

#define X_LT_BORDER      60
#define Y_LT_BORDER      40
#define BORDER_WIDTH     500
#define BORDER_HEIGHT    70

#define TIME_WIDTH       115
#define DIGIT_WIDTH      26
#define DIGIT_HEIGHT     50
#define MARGIN           4


#define CHANGE_STATE( app ) { \
        state_title_init( (app) ); \
        (app)->next_state = STATE_TITLE; \
        return 1; \
}

#define SAVE( app ) { \
        if ( ! state_save( (app) ) ) { \
                (app)->select_screen.effect_savedone_frame_count = \
                        state_get_framecount_effect_message(); \
	} \
	else { \
		(app)->select_screen.effect_savefail_frame_count = \
			state_get_framecount_effect_message(); \
        } \
}


static void state_select_draw_level( SpaceMazeApp* app );
static void state_select_draw_time( SpaceMazeApp* app, int level, int index_number, int y_center );
static void state_select_draw_border( SpaceMazeApp* app, int level );
static void state_select_draw_save( SpaceMazeApp* app );
static void state_select_draw_back( SpaceMazeApp* app );
static void state_select_focus_level( SpaceMazeApp* app, int level );
static void state_select_focus_save( SpaceMazeApp* app );
static void state_select_unfocus_save( SpaceMazeApp* app );
static void state_select_focus_back( SpaceMazeApp* app );
static void state_select_unfocus_back( SpaceMazeApp* app );
static int  state_select_within_level( SpaceMazeApp* app, int level, int x, int y );
static int  state_select_within_save( SpaceMazeApp* app, int x, int y );
static int  state_select_within_back( SpaceMazeApp* app, int x, int y );
static int  state_select_key_pressed( void* data );
static int  state_select_mouse_moved( void* data );
static int  state_select_mouse_pressed( void* data );
static void state_select_check_mouse_position( SpaceMazeApp* app );


/*!
 * Initializes select screen related variables of application context.
 * @param app [i/o] Application context.
 */
void
state_select_init( SpaceMazeApp* app )
{
	memset( &( app->select_screen ), 0, sizeof( SelectScreen ) );
	app->select_screen.previous_level = -1;
	state_select_focus_level( app, MAP_MIN_LEVEL );
	state_select_set_event_handler( app );
	app->select_screen.initialized = 1;
}


/*!
 * Sets event handler.
 */
void
state_select_set_event_handler( SpaceMazeApp* app )
{
	if ( app != NULL ) {
		rlib_gt_clear_event_handler( &( app->gt ) );

		rlib_gt_add_event_handler( &( app->gt ),
			RL_GT_KEY_PRESSED, state_select_key_pressed, ( void* )app );
		rlib_gt_add_event_handler( &( app->gt ),
			RL_GT_MOUSE_MOVED, state_select_mouse_moved, ( void* )app );
		rlib_gt_add_event_handler( &( app->gt ),
			RL_GT_MOUSE_PRESSED, state_select_mouse_pressed, ( void* )app );
	}
}


/*!
 * This function will be called within main loop.
 *
 * - Checks keyboard / mouse event ( These will be done within rlib_gt_process_event() ).
 *   - If mouse is within map level text area or back text area, puts focus on the text.
 *   - If up / down key is pressed, puts focus on appropriate map level text or back text.
 *   - If enter key or left button of mouse is pressed 
 *     within map level text area, changes game state to STATE_PLAYING.
 *   - If enter key or left button of mouse is pressed
 *     within back text area, change game state to STATE_TITLE.
 * - Updates star object's information.
 *
 * @param app [i/o] Application context.
 * @retval Next game state.
 */
int
state_select_update( SpaceMazeApp* app )
{
	if ( app == NULL ) {
		return STATE_INVALID;
	}

	if ( app->select_screen.is_selected ) {
		app->select_screen.effect_selected_frame_count ++;
	}
	else {
		if ( app->select_screen.effect_savefail_frame_count == 0 &&
		     app->select_screen.effect_savedone_frame_count == 0 ) {

			if ( rlib_gt_process_event( &( app->gt ) ) != 0 ) {
				return app->next_state;
			}

			state_select_check_mouse_position( app );
		}

		if ( app->select_screen.effect_savefail_frame_count > 0 ) {
			app->select_screen.effect_savefail_frame_count --;
		}
		else if ( app->select_screen.effect_savedone_frame_count > 0 ) {
			app->select_screen.effect_savedone_frame_count --;
		}
	}

	if ( app->select_screen.effect_selected_frame_count >= 
	     ( FRAME_COUNT_EFFECT_SELECTED * state_get_fps_ratio() ) ) {
		int i;
		for ( i = 0; i <= MAP_MAX_LEVEL; i ++ ) {
			if ( app->select_screen.is_maplevel_focused[i] ) {
				if ( i == MAP_MAX_LEVEL ) {
					state_boss_init( app, i );
					return STATE_BOSS;
				}
				else {
					state_playing_init( app, i );
					return STATE_PLAYING;
				}
			}
		}
	}

	UPDATE_STARS( app );
	return app->game_state;
}


/*!
 * Draws select screen.
 * @param app [in] Application context.
 */
void
state_select_draw( SpaceMazeApp* app )
{
	if ( app == NULL ) {
		return;
	}

	bg_draw( &( app->gt ) );
	DRAW_STARS( app );
	state_select_draw_level( app );
	state_select_draw_save( app );
	state_select_draw_back( app );

	if ( app->select_screen.effect_savefail_frame_count > 0 ) {
		DRAW_IMAGE( app, INDEX_IMAGE_SAVE_FAIL, X_CENTER_MESSAGE, Y_CENTER_MESSAGE );
	}
	else if ( app->select_screen.effect_savedone_frame_count > 0 ) {
		DRAW_IMAGE( app, INDEX_IMAGE_SAVE_DONE, X_CENTER_MESSAGE, Y_CENTER_MESSAGE );
	}
}


static void
state_select_draw_level( SpaceMazeApp* app )
{
	int y_center = Y_CENTER_LEVEL;
	int index_level;
	int index_number;
	int index_time;
	int i;

	for ( i = 0; i <= MAP_MAX_LEVEL; i ++ ) {
		if ( ! app->select_screen.is_back_focused && 
		     ! app->select_screen.is_save_focused &&
		     app->select_screen.is_maplevel_focused[i] ) {
			index_level  = INDEX_IMAGE_LEVEL0;
			index_number = INDEX_IMAGE_NUMBERS0_0;
			index_time   = INDEX_IMAGE_TIME0;
		}
		else if ( i == 0 || app->record.keys[ i - 1 ] ) {
			index_level  = INDEX_IMAGE_LEVEL2;
			index_number = INDEX_IMAGE_NUMBERS2_0;
			index_time   = INDEX_IMAGE_TIME2;
		}
		else {
			continue;
			/*
			 * At first, I was going to show text of all the levels, but I 
			 * changed my mind to show only cleared level and one uncleared level. 
			 * I think this will make the game more thrilling !
			 */
#if 0
			index_level  = INDEX_IMAGE_LEVEL1;
			index_number = INDEX_IMAGE_NUMBERS1_0;
			index_time   = INDEX_IMAGE_TIME1;
#endif
		}

		DRAW_IMAGE( app, index_level, X_CENTER_LEVEL, y_center );
		DRAW_IMAGE( app, index_number + ( i + 1 ), X_CENTER_DIGIT, y_center );
		DRAW_IMAGE( app, index_time, X_CENTER_TIME, y_center );

		state_select_draw_time( app, i, index_number, y_center );
		state_select_draw_border( app, i );
		y_center += Y_MARGIN_LEVEL;
	}
}


static void
state_select_draw_time( SpaceMazeApp* app, int level, int index_number, int y_center )
{
#define DIGIT_NUM 5
#define OFFSET_DM 10

	int digit[ DIGIT_NUM ];
	int x_center = X_CENTER_TIME + TIME_WIDTH / 2 + MARGIN * 2;
	int i;

	digit[0] = app->record.mins[ level ] / 10;
	digit[1] = app->record.mins[ level ] - digit[0] * 10;
	digit[2] = ':';
	digit[3] = app->record.secs[ level ] / 10;
	digit[4] = app->record.secs[ level ] - digit[3] * 10;

	if ( digit[0] == 0 && digit[1] == 0 && digit[3] == 0 && digit[4] == 0 ) {
		return;
	}

	for ( i = 0; i < DIGIT_NUM; i ++ ) {
		if ( digit[i] == ':' ) {
			DRAW_IMAGE( app, index_number + OFFSET_DM, x_center + MARGIN, y_center );
		}
		else {
			DRAW_IMAGE( app, index_number + digit[i],  x_center + MARGIN, y_center );
		}

		x_center += DIGIT_WIDTH;
	}
}


static void
state_select_draw_border( SpaceMazeApp* app, int level )
{
	if ( app->select_screen.is_maplevel_focused[ level ] ) {
		if ( app->select_screen.is_selected ) {
			if ( app->select_screen.effect_selected_frame_count % 
			     ( int )state_get_framecount_effect_blink() <= 
			         ( int )state_get_framecount_effect_blink() / 2 ) {
				return;
			}

			rlib_gt_set_foreground_color( &( app->gt ), RL_GT_MAX_RGB, 0, 0 );
		}
		else if ( app->select_screen.is_back_focused || app->select_screen.is_save_focused ) {
			rlib_gt_set_foreground_color( &( app->gt ), 0, 0, RL_GT_MAX_RGB );
		}
		else {
			rlib_gt_set_foreground_color( &( app->gt ), RL_GT_MAX_RGB, 0, 0 );
		}

		rlib_gt_draw_rectangle( 
			&( app->gt ), 
			X_LT_BORDER, 
			Y_LT_BORDER + Y_MARGIN_LEVEL * level,
			BORDER_WIDTH,
			BORDER_HEIGHT
			);
	}
}


static void
state_select_draw_save( SpaceMazeApp* app )
{
	int index;

	if ( app->select_screen.is_save_focused ) {
		index = INDEX_IMAGE_SAVE_FOCUS;
	}
	else {
		index = INDEX_IMAGE_SAVE_NOFOCUS;
	}

	DRAW_IMAGE( app, index, X_CENTER_SAVE, Y_CENTER_SAVE );
}


static void
state_select_draw_back( SpaceMazeApp* app )
{
	int index;

	if ( app->select_screen.is_back_focused ) {
		index = INDEX_IMAGE_BACK_FOCUS;
	}
	else {
		index = INDEX_IMAGE_BACK_NOFOCUS;
	}

	DRAW_IMAGE( app, index, X_CENTER_BACK, Y_CENTER_BACK );
}


static void
state_select_focus_level( SpaceMazeApp* app, int level )
{
	int i;

	if ( level == 0 || app->record.keys[ level - 1 ] ) {
		for ( i = 0; i <= MAP_MAX_LEVEL; i ++ ) {
			if ( i != level ) {
				app->select_screen.is_maplevel_focused[i] = 0;
			}
			else {
				app->select_screen.is_maplevel_focused[i] = 1;

				if ( app->select_screen.previous_level != i && app->select_screen.initialized ) {
					rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_SELECT ], 0 );
					app->select_screen.previous_level = i;
				}
			}
		}
	}
}


static void
state_select_focus_save( SpaceMazeApp* app )
{
	if ( ! app->select_screen.is_save_focused ) {
		rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_SELECT ], 0 );		
	}

	app->select_screen.is_save_focused = 1;
	app->select_screen.is_back_focused = 0;
}


static void
state_select_unfocus_save( SpaceMazeApp* app )
{
	app->select_screen.is_save_focused = 0;
}


static void
state_select_focus_back( SpaceMazeApp* app )
{
	if ( ! app->select_screen.is_back_focused ) {
		rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_SELECT ], 0 );
	}

	app->select_screen.is_save_focused = 0;
	app->select_screen.is_back_focused = 1;
}


static void
state_select_unfocus_back( SpaceMazeApp* app )
{
	app->select_screen.is_back_focused = 0;
}


static int
state_select_within_level( SpaceMazeApp* app, int level, int x, int y )
{
	int lt_x = X_LT_BORDER;
	int lt_y = Y_LT_BORDER + Y_MARGIN_LEVEL * level;

	return rlib_rect_within_low( lt_x, lt_y, BORDER_WIDTH, BORDER_HEIGHT, x, y );
}


static int
state_select_within_save( SpaceMazeApp* app, int x, int y )
{
	int width;
	int height;
	int lt_x;
	int lt_y;

	RL_GT_LEFTTOP( 
		INDEX_IMAGE_SAVE_FOCUS, 
		&( app->gt ), 
		X_CENTER_SAVE,
		Y_CENTER_SAVE,
		lt_x, 
		lt_y, 
		width, 
		height );

	return rlib_rect_within_low( lt_x, lt_y, width, height, x, y );
}


static int
state_select_within_back( SpaceMazeApp* app, int x, int y )
{
	int width;
	int height;
	int lt_x;
	int lt_y;

	RL_GT_LEFTTOP( 
		INDEX_IMAGE_BACK_FOCUS, 
		&( app->gt ), 
		X_CENTER_BACK,
		Y_CENTER_BACK,
		lt_x, 
		lt_y, 
		width, 
		height );

	return rlib_rect_within_low( lt_x, lt_y, width, height, x, y );
}


static int
state_select_key_pressed( void* data )
{
	int i;

	SpaceMazeApp* app = ( SpaceMazeApp* )data;
	app->mode_key = 1;

	if ( rlib_gt_key_pressed( &( app->gt ), RL_GT_KEY_ESC ) ) {
		rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_ESC ], 0 );
		state_title_init( app );
		app->next_state = STATE_TITLE;
		return 1;
	}

	if ( rlib_gt_key_pressed( &( app->gt ), RL_GT_KEY_DOWN ) ) {
		if ( ! app->select_screen.is_save_focused && ! app->select_screen.is_back_focused ) {
			for ( i = 0; i < MAP_MAX_LEVEL; i ++ ) {
				if ( app->select_screen.is_maplevel_focused[i] ) {
					state_select_focus_level( app, i + 1 );
					break;
				}
			}
		}
	}
	
	if ( rlib_gt_key_pressed( &( app->gt ), RL_GT_KEY_UP ) ) {
		if ( ! app->select_screen.is_save_focused && ! app->select_screen.is_back_focused ) {
			for ( i = 1; i <= MAP_MAX_LEVEL; i ++ ) {
				if ( app->select_screen.is_maplevel_focused[i] ) {
					state_select_focus_level( app, i - 1 );
					break;
				}
			}
		}
	}

	if ( rlib_gt_key_pressed( &( app->gt ), RL_GT_KEY_RIGHT ) ) {
		if ( app->select_screen.is_save_focused ) {
			state_select_focus_back( app );
		}
		else if ( ! app->select_screen.is_back_focused ) {
			state_select_focus_save( app );
		}
	}

	if ( rlib_gt_key_pressed( &( app->gt ), RL_GT_KEY_LEFT ) ) {
		if ( app->select_screen.is_back_focused ) {
			state_select_focus_save( app );
		}
		else if ( app->select_screen.is_save_focused ) {
			state_select_unfocus_save( app );
			rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_SELECT ], 0 );
		}
	}

	if ( rlib_gt_key_pressed( &( app->gt ), RL_GT_KEY_RETURN ) ) {
		if ( app->select_screen.is_back_focused ) {
			rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_CLICK ], 0 );
			CHANGE_STATE( app );
		}
		else if ( app->select_screen.is_save_focused ) {
			rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_CLICK ], 0 );
			SAVE( app );
		}
		else {
			for ( i = 0; i <= MAP_MAX_LEVEL; i ++ ) {
				if ( app->select_screen.is_maplevel_focused[i] ) {
					rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_CLICK ], 0 );
					app->select_screen.is_selected = 1;
					break;
				}
			}
		}
	}

	return 0;
}


static int
state_select_mouse_moved( void* data )
{
	SpaceMazeApp* app = ( SpaceMazeApp* )data;
	app->mode_key = 0;
	return 0;
}


static int
state_select_mouse_pressed( void* data )
{
	int x;
	int y;
	int i;

	SpaceMazeApp* app = ( SpaceMazeApp* )data;
	app->mode_key = 0;

	if ( rlib_gt_mouse_pressed( &( app->gt ), RL_GT_MOUSE_LEFT, &x, &y ) ) {
		if ( app->select_screen.is_back_focused &&
		     state_select_within_back( app, x, y ) ) {

			rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_CLICK ], 0 );
			CHANGE_STATE( app );
		}
		else if ( app->select_screen.is_save_focused &&
			  state_select_within_save( app, x, y ) ) {

			rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_CLICK ], 0 );
			SAVE( app );
		}
		else {
			for ( i = 0; i <= MAP_MAX_LEVEL; i ++ ) {
				if ( app->select_screen.is_maplevel_focused[i] &&
				     state_select_within_level( app, i, x, y ) ) {
					rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_CLICK ], 0 );
					app->select_screen.is_selected = 1;
					break;
				}
			}
		}
	}

	return 0;
}


static void
state_select_check_mouse_position( SpaceMazeApp* app )
{
	int x;
	int y;
	int i;

	if ( app->mode_key ) {
		return;
	}

	rlib_x_mouse_query_position( &( app->gt ), &x, &y );

	if ( state_select_within_back( app, x, y ) ) {
		state_select_focus_back( app );
	}
	else if ( state_select_within_save( app, x, y ) ) {
		state_select_focus_save( app );
	}
	else {
		for ( i = 0; i <= MAP_MAX_LEVEL; i ++ ) {
			if ( state_select_within_level( app, i, x, y ) ) {
				if ( i == 0 || app->record.keys[ i - 1 ] ) {
					state_select_unfocus_back( app );
					state_select_unfocus_save( app );
					state_select_focus_level( app, i );
				}
			}
		}
	}
}


/* End of state_select.c */
