/*
 * Copyright (c) 2005,2006 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief Handles title screen.
 *
 * "$Id: state_title.c,v 1.2 2006/09/21 15:30:16 ryo Exp $"
 */

#include <string.h>

#include "RLgt.h"
#include "RLrect.h"
#include "RLsound.h"
#include "bg.h"
#include "spacemaze.h"
#include "state.h"


#define FRAME_COUNT_EFFECT_SELECTED 150

#define Y_CENTER_TITLE     100 
#define Y_CENTER_COPYRIGHT 430
#define Y_CENTER_START     240
#define Y_CENTER_LOAD      290
#define Y_CENTER_EXIT      340

#define X_CENTER_MESSAGE   320
#define Y_CENTER_MESSAGE   200


#define LOAD( app ) { \
        if ( state_load( (app) ) ) { \
                (app)->title_screen.effect_loadfail_frame_count = \
                        state_get_framecount_effect_message(); \
        } \
        else { \
                (app)->title_screen.is_selected = 1; \
        } \
}


static void state_title_draw_starttext( SpaceMazeApp* app );
static void state_title_draw_loadtext( SpaceMazeApp* app );
static void state_title_draw_exittext( SpaceMazeApp* app );
static void state_title_focus_start( SpaceMazeApp* app );
static void state_title_focus_load( SpaceMazeApp* app );
static void state_title_focus_exit( SpaceMazeApp* app );
static int  state_title_within_start( SpaceMazeApp* app, int x, int y );
static int  state_title_within_load( SpaceMazeApp* app, int x, int y );
static int  state_title_within_exit( SpaceMazeApp* app, int x, int y );
static int  state_title_key_pressed( void* data );
static int  state_title_mouse_moved( void* data );
static int  state_title_mouse_pressed( void* data );
static void state_title_check_mouse_position( SpaceMazeApp* app );


/*!
 * Initializes title screen related variables of application context.
 * @param app [i/o] Application context.
 */
void
state_title_init( SpaceMazeApp* app )
{
	memset( &( app->record ), 0, sizeof( Record ) );
	memset( &( app->title_screen ), 0, sizeof( TitleScreen ) );
	state_title_focus_start( app );
	state_title_set_event_handler( app );
	app->title_screen.initialized = 1;
}


/*!
 * Sets event handler.
 */
void
state_title_set_event_handler( SpaceMazeApp* app )
{
	if ( app != NULL ) {
		rlib_gt_clear_event_handler( &( app->gt ) );

		rlib_gt_add_event_handler( &( app->gt ), 
			RL_GT_KEY_PRESSED, state_title_key_pressed, ( void* )app );
		rlib_gt_add_event_handler( &( app->gt ), 
			RL_GT_MOUSE_MOVED, state_title_mouse_moved, ( void* )app );
		rlib_gt_add_event_handler( &( app->gt ), 
			RL_GT_MOUSE_PRESSED, state_title_mouse_pressed, ( void* )app );
	}
}


/*!
 * This function will be called within main loop.
 *
 * - Checks keyboard / mouse event ( These will be done within rlib_gt_process_event() ).
 *   - If mouse is within start / load / exit text area, puts focus on the text.
 *   - If up / down key is pressed, puts focus on start / load / exit text.
 *   - If enter key or left button of mouse is pressed 
 *     within start text area, changes game state to STATE_SELECT.
 *   - If enter key or left button of mouse is pressed 
 *     within load text area, changes game state to STATE_RECORD.
 *   - If enter key or left button of mouse is pressed
 *     within exit text area, exits main loop.
 * - Updates all object's information.
 *
 * @param app [i/o] Application context.
 * @retval Next game state.
 */
int
state_title_update( SpaceMazeApp* app )
{
	if ( app == NULL ) {
		return STATE_INVALID;
	}

	if ( app->title_screen.is_selected ) {
		app->title_screen.effect_selected_frame_count ++;
	}
	else {
		if ( app->title_screen.effect_loadfail_frame_count == 0 ) {
			if ( rlib_gt_process_event( &( app->gt ) ) != 0 ) {
				return app->next_state;
			}

			state_title_check_mouse_position( app );
		}

		if ( app->title_screen.effect_loadfail_frame_count > 0 ) {
			app->title_screen.effect_loadfail_frame_count --;
		}
	}

	if ( app->title_screen.effect_selected_frame_count >= 
	     ( FRAME_COUNT_EFFECT_SELECTED * state_get_fps_ratio() ) ) {
		state_select_init( app );
		return STATE_SELECT;
	}

	UPDATE_STARS( app );
	return app->game_state;
}


/*!
 * Draws title screen.
 * @param app [in] Application context.
 */
void
state_title_draw( SpaceMazeApp* app )
{
	double center_x = SPACEMAZE_SCREEN_WIDTH / 2;
	double center_y;

	if ( app == NULL ) {
		return;
	}

	bg_draw( &( app->gt ) );
	DRAW_STARS( app );

	center_y = Y_CENTER_TITLE;
	DRAW_IMAGE( app, INDEX_IMAGE_TITLE, center_x, center_y );

	state_title_draw_starttext( app );
	state_title_draw_loadtext( app );
	state_title_draw_exittext( app );

	center_y = Y_CENTER_COPYRIGHT;
	DRAW_IMAGE( app, INDEX_IMAGE_COPYRIGHT, center_x, center_y );

	if ( app->title_screen.effect_loadfail_frame_count > 0 ) {
		DRAW_IMAGE( app, INDEX_IMAGE_LOAD_FAIL, X_CENTER_MESSAGE, Y_CENTER_MESSAGE );
	}
}


static void
state_title_draw_starttext( SpaceMazeApp* app )
{
	int index;
	
	if ( app->title_screen.is_selected && app->title_screen.is_start_focused ) {
		if ( app->title_screen.effect_selected_frame_count % 
		     ( int )state_get_framecount_effect_blink() <= ( int )state_get_framecount_effect_blink() / 2 ) {

			return;
		}

		index = INDEX_IMAGE_START_FOCUS;
	}
	else if ( app->title_screen.is_start_focused ) {
		index = INDEX_IMAGE_START_FOCUS;
	}
	else {
		index = INDEX_IMAGE_START_NOFOCUS;
	}

	DRAW_IMAGE( app, index, SPACEMAZE_SCREEN_WIDTH / 2, Y_CENTER_START );
}


static void
state_title_draw_loadtext( SpaceMazeApp* app )
{
	int index;
	
	if ( app->title_screen.is_selected && app->title_screen.is_load_focused ) {
		if ( app->title_screen.effect_selected_frame_count % 
		     ( int )state_get_framecount_effect_blink() <= 
		         ( int )state_get_framecount_effect_blink() / 2 ) {

			return;
		}

		index = INDEX_IMAGE_LOAD_FOCUS;
	}
	else if ( app->title_screen.is_load_focused ) {
		index = INDEX_IMAGE_LOAD_FOCUS;
	}
	else {
		index = INDEX_IMAGE_LOAD_NOFOCUS;
	}

	DRAW_IMAGE( app, index, SPACEMAZE_SCREEN_WIDTH / 2, Y_CENTER_LOAD );
}


static void
state_title_draw_exittext( SpaceMazeApp* app )
{
	int index;

	if ( app->title_screen.is_exit_focused ) {
		index = INDEX_IMAGE_EXIT_FOCUS;
	}
	else {
		index = INDEX_IMAGE_EXIT_NOFOCUS;
	}

	DRAW_IMAGE( app, index, SPACEMAZE_SCREEN_WIDTH / 2, Y_CENTER_EXIT );
}


static void
state_title_focus_start( SpaceMazeApp* app )
{
	if ( ! app->title_screen.is_start_focused && app->title_screen.initialized ) {
		rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_SELECT ], 0 );
	}

	app->title_screen.is_start_focused = 1;
	app->title_screen.is_load_focused  = 0;
	app->title_screen.is_exit_focused  = 0;
}


static void
state_title_focus_load( SpaceMazeApp* app )
{
	if ( ! app->title_screen.is_load_focused ) {
		rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_SELECT ], 0 );
	}

	app->title_screen.is_start_focused = 0;
	app->title_screen.is_load_focused  = 1;
	app->title_screen.is_exit_focused  = 0;
}


static void
state_title_focus_exit( SpaceMazeApp* app )
{
	if ( ! app->title_screen.is_exit_focused ) {
		rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_SELECT ], 0 );
	}

	app->title_screen.is_start_focused = 0;
	app->title_screen.is_load_focused  = 0;
	app->title_screen.is_exit_focused  = 1;
}


static int
state_title_within_start( SpaceMazeApp* app, int x, int y )
{
	int width;
	int height;
	int lt_x;
	int lt_y;

	RL_GT_LEFTTOP( 
		INDEX_IMAGE_START_FOCUS, 
		&( app->gt ), 
		SPACEMAZE_SCREEN_WIDTH / 2,
		Y_CENTER_START, 
		lt_x, 
		lt_y, 
		width, 
		height );

	return rlib_rect_within_low( lt_x, lt_y, width, height, x, y );
}


static int
state_title_within_load( SpaceMazeApp* app, int x, int y )
{
	int width;
	int height;
	int lt_x;
	int lt_y;

	RL_GT_LEFTTOP( 
		INDEX_IMAGE_LOAD_FOCUS, 
		&( app->gt ), 
		SPACEMAZE_SCREEN_WIDTH / 2,
		Y_CENTER_LOAD,
		lt_x, 
		lt_y, 
		width, 
		height );

	return rlib_rect_within_low( lt_x, lt_y, width, height, x, y );
}


static int
state_title_within_exit( SpaceMazeApp* app, int x, int y )
{
	int width;
	int height;
	int lt_x;
	int lt_y;

	RL_GT_LEFTTOP( 
		INDEX_IMAGE_EXIT_FOCUS, 
		&( app->gt ), 
		SPACEMAZE_SCREEN_WIDTH / 2,
		Y_CENTER_EXIT,
		lt_x, 
		lt_y, 
		width, 
		height );

	return rlib_rect_within_low( lt_x, lt_y, width, height, x, y );
}


static int
state_title_key_pressed( void* data )
{
	SpaceMazeApp* app = ( SpaceMazeApp* )data;
	app->mode_key = 1;

	if ( rlib_gt_key_pressed( &( app->gt ), RL_GT_KEY_ESC ) ) {
		rlib_gt_stop( &( app->gt ), 0 );
		return 1;
	}

	if ( rlib_gt_key_pressed( &( app->gt ), RL_GT_KEY_DOWN ) ) {
		if ( app->title_screen.is_start_focused ) {
			state_title_focus_load( app );
		}
		else if ( app->title_screen.is_load_focused ) {
			state_title_focus_exit( app );
		}
	}
	
	if ( rlib_gt_key_pressed( &( app->gt ), RL_GT_KEY_UP ) ) {
		if ( app->title_screen.is_exit_focused ) {
			state_title_focus_load( app );
		}
		else if ( app->title_screen.is_load_focused ) {
			state_title_focus_start( app );
		}
	}

	if ( rlib_gt_key_pressed( &( app->gt ), RL_GT_KEY_RETURN ) ) {
		if ( app->title_screen.is_start_focused ) {
			rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_CLICK ], 0 );
			app->title_screen.is_selected = 1;
		}
		else if ( app->title_screen.is_load_focused ) {
			rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_CLICK ], 0 );
			LOAD( app );
		}
		else if ( app->title_screen.is_exit_focused ) {
			rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_CLICK ], 0 );
			rlib_gt_stop( &( app->gt ), 0 );
			return 1;
		}
	}

	return 0;
}


static int
state_title_mouse_moved( void* data )
{
	SpaceMazeApp* app = ( SpaceMazeApp* )data;
	app->mode_key = 0;
	return 0;
}


static int
state_title_mouse_pressed( void* data )
{
	int x;
	int y;

	SpaceMazeApp* app = ( SpaceMazeApp* )data;
	app->mode_key = 0;

	if ( rlib_gt_mouse_pressed( &( app->gt ), RL_GT_MOUSE_LEFT, &x, &y ) ) {
		if ( app->title_screen.is_start_focused && state_title_within_start( app, x, y ) ) {
			rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_CLICK ], 0 );
			app->title_screen.is_selected = 1;
		}
		else if ( app->title_screen.is_load_focused && state_title_within_load( app, x, y ) ) {
			rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_CLICK ], 0 );
			LOAD( app );
		}
		else if ( app->title_screen.is_exit_focused && state_title_within_exit( app, x, y ) ) {
			rlib_sound_play( &( app->sound ), app->wave_index[ INDEX_WAVE_CLICK ], 0 );
			rlib_gt_stop( &( app->gt ), 0 );
			return 1;
		}
	}

	return 0;
}


static void
state_title_check_mouse_position( SpaceMazeApp* app )
{
	int x;
	int y;

	if ( app->mode_key ) {
		return;
	}

	rlib_x_mouse_query_position( &( app->gt ), &x, &y );

	if ( state_title_within_start( app, x, y ) ) {
		state_title_focus_start( app );
	}
	else if ( state_title_within_load( app, x, y ) ) {
		state_title_focus_load( app );
	}
	else if ( state_title_within_exit( app, x, y ) ) {
		state_title_focus_exit( app );
	}
}


/* End of state_title.c */
