/*
 * Copyright (c) 2005,2006 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief Handles rectangle data.
 *
 * $Id: RLrect.c,v 1.1.1.1 2006/07/22 18:07:56 ryo Exp $
 */

#include <stdio.h>
#include "RLrect.h"


/*!
 * Initializes a rectangle data struct. 
 * "rect_ptr" must not be NULL.
 * When "rect_ptr" is NULL, this function does nothing.
 *
 * @param rect_ptr [i/o] Rectangle data struct. Must not be NULL.
 */
void
rlib_rect_init( RLrect* rect_ptr, int lefttop_x, int lefttop_y, int width, int height )
{
	if ( rect_ptr == NULL ) {
		return;
	}

	rect_ptr->lefttop_x = lefttop_x;
	rect_ptr->lefttop_y = lefttop_y;
	rect_ptr->width     = width;
	rect_ptr->height    = height;
}


/*!
 * Checks if the specified point is within the specified rectangle.
 * This function uses rlib_rect_within_low internally.
 *
 * @param rect_ptr [in] Rectangle data.
 * @param x        [in] x position of the point to check.
 * @param y        [in] y position of the point to check.
 *
 * @retval  1 Specified point is within specified rectangle.
 * @retval  0 Specified point is outside of specified rectangle, 
 *            or rect_ptr is NULL.
 */
int
rlib_rect_within( RLrect* rect_ptr, int x, int y )
{
	if ( rect_ptr == NULL ) {
		return 0;
	}

	return rlib_rect_within_low( 
		rect_ptr->lefttop_x,
		rect_ptr->lefttop_y,
		rect_ptr->width,
		rect_ptr->height,
		x,
		y );
}


/*!
 * Checks if the specified point is within the specified rectangle.
 *
 * @param lefttop_x [in] Left top x position of the rectangle.
 * @param lefttop_y [in] Left top y position of the rectangle.
 * @param width     [in] Width  of the rectangle.
 * @param height    [in] Height of the rectangle.
 * @param x         [in] x position of the point to check.
 * @param y         [in] y position of the point to check.
 *
 * @retval  1  Specified point is within the specified rectangle.
 * @retval  0  Specified point is outside of the specified rectangle, 
 *             or rect_ptr is NULL.
 */
int
rlib_rect_within_low( 
	int lefttop_x, int lefttop_y, int width, int height, int x, int y )
{
	if ( x < lefttop_x || x > lefttop_x + width ) {
		return 0;
	}

	if ( y < lefttop_y || y > lefttop_y + height ) {
		return 0;
	}

	return 1;
}


/*!
 * Checks if the specified two rectangles overlap.
 *
 * @param rect1_ptr [in] A rectangle.
 * @param rect2_ptr [in] A rectangle.
 *
 * @retval 1 Specified rectangles overlap.
 * @retval 0 Specified rectangles don't overlap, 
 *           or some of arguments are NULL.
 */
int
rlib_rect_overlap( RLrect* rect1_ptr, RLrect* rect2_ptr )
{
	if ( rect1_ptr == NULL || rect2_ptr == NULL ) {
		return 0;
	}

	if ( rect1_ptr->lefttop_x <= rect2_ptr->lefttop_x - rect1_ptr->width ||
	     rect1_ptr->lefttop_x >= rect2_ptr->lefttop_x + rect2_ptr->width ) {

		return 0;
	}

	if ( rect1_ptr->lefttop_y <= rect2_ptr->lefttop_y - rect1_ptr->height ||
	     rect1_ptr->lefttop_y >= rect2_ptr->lefttop_y + rect2_ptr->height ) {

		return 0;
	}

	return 1;
}


/* End of RLrect.c */
