/*
 * Copyright (c) 2002-2007 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief Utility functions for handling RIFF / WAVE file.
 *
 * $Id: RLwave.c,v 1.2 2007/11/04 11:41:12 ryo Exp $
 */

#include <errno.h>
#include <stdio.h>
#include <string.h>

#include "RLerrno.h"
#include "RLwave.h"


/*
 * static function prototype.
 */
static int 
rlib_wave_check_type( RLWAVE* wavep, int* err );

static int
rlib_wave_read_format( RLWAVE* wavep, int* err );

static int
rlib_wave_find_pcm( RLWAVE* wavep, int* err );


/*!
 * Opens specified file by "filepath_ptr".
 * When executed with RLWAVE_READ mode, No check for its format 
 * would be executed, it would be done when you call rlib_wave_read_header().
 * All arguments must not be NULL.
 *
 * @param wavep        [i/o] WAVE data struct. Must not be NULL.
 * @param filepath_ptr [in]  WAVE file path. Must not be NULL.
 * @param mode         [in]  File open mode. RLWAVE_READ will be used as default value.
 * @param err          [out] errno will be saved. Must not be NULL.
 *
 * @retval  0            Successfully processed.
 * @retval  RLIB_ENOFILE Specified file could not be found.
 * @retval  RLIB_ERR     C Library function returns error.
 *                       "err" will be set to errno, so "err" must not be NULL.
 *
 * @retval  RLIB_EFAULT  "wavep" or "filepath_ptr" or "err" is NULL.
 */
int
rlib_wave_open( RLWAVE* wavep, const char* filepath_ptr, int mode, int* err )
{
	if ( wavep == NULL || filepath_ptr == NULL || err == NULL ) {
		return RLIB_EFAULT;
	}

	memset( wavep, 0, sizeof( RLWAVE ) );

	if ( mode == RLWAVE_WRITE ) {
		wavep->fp = fopen( filepath_ptr, "w" );
	}
	else {
		wavep->fp = fopen( filepath_ptr, "r" );
	}

	if ( wavep->fp == NULL ) {
		*err = errno;

		switch ( *err ) {
			case ENOENT :
			case ENOTDIR:
				return RLIB_ENOFILE;

			default:
				return RLIB_ERR;
		}
	}

	return 0;
}


/*!
 * Reads header of the wave file specified by rlib_wave_open().
 * Information of wave file would be saved into struct specified by "wavep".
 * "wavep" and "err" must not be NULL.
 *
 * @param wavep [i/o] WAVE data struct. Must not be NULL.
 * @param err   [out] errno will be saved. Must not be NULL.
 *
 * @retval  0                  Successfully processed.
 * @retval  RLIB_ERR           C Library function returns error.
 *                             "err" will be set to errno, so "err" must not be NULL.
 *
 * @retval  RLIB_EOF           Unexpected EOF meets.
 * @retval  RLIB_EFMT          Invalid format.
 * @retval  RLIB_ENOTSUPPORTED Format of specified WAVE file is not supported.
 *
 * @retval  RLIB_EFAULT        "wavep" or "err"  is NULL.
 *                             It also returns this when wavep->fp is NULL.
 */
int
rlib_wave_read_header( RLWAVE* wavep, int* err )
{
	int ret;

	if ( wavep == NULL || err == NULL || wavep->fp == NULL ) {
		return RLIB_EFAULT;
	}

	if ( ( ret = rlib_wave_check_type( wavep, err ) )  != 0 ) {
		return ret;
	}

	if ( ( ret = rlib_wave_read_format( wavep, err ) ) != 0 ) {
		return ret;
	}

	if ( ( ret = rlib_wave_find_pcm( wavep, err ) ) != 0 ) {
		return ret;
	}

	return 0;
}


#define RLWAVE_LABEL_RIFF "RIFF"
#define RLWAVE_LABEL_WAVE "WAVE"
#define RLWAVE_LABEL_FMT  "fmt "
#define RLWAVE_LABEL_DATA "data"


/*!
 * Writes header of the wave file specified in rlib_wave_open().
 * Set the member value of "wavep", then those values will be written
 * in the form of wave file header.
 * "wavep" and "err" must not be NULL.
 *
 * @param wavep   [i/o] WAVE data struct. Must not be NULL.
 * @param err     [out] errno will be saved. Must not be NULL.
 *
 * @retval  0                  Successfully processed.
 * @retval  RLIB_ERR           C Library function returns error.
 *                             "err" will be set to errno, so "err" must not be NULL.
 *
 * @retval  RLIB_EFAULT        "wavep" or "err"  is NULL.
 *                             It also returns this when wavep->fp is NULL.
 */
int
rlib_wave_write_header( RLWAVE* wavep, int* err )
{
	int fmthead_len;
	int wave_len;
	int byte_sec;
	int byte_dat;
	int shortval;

	if ( wavep == NULL || err == NULL || wavep->fp == NULL ) {
		return RLIB_EFAULT;
	}

	fmthead_len = RLWAVE_FMTHEAD_LENGTH;
	shortval    = ( short )fmthead_len;
	wave_len    = RLWAVE_WAVHEAD_LENGTH + fmthead_len + RLWAVE_PCMHEAD_LENGTH + wavep->len;
	byte_sec    = wavep->rate * wavep->bits / 8;
	byte_dat    = wavep->bits / 8;

	if ( fwrite( RLWAVE_LABEL_RIFF, strlen( RLWAVE_LABEL_RIFF ), 1, wavep->fp ) < 1 ||
	     fwrite( &wave_len,       4, 1, wavep->fp ) < 1 ||
	     fwrite( RLWAVE_LABEL_WAVE, strlen( RLWAVE_LABEL_WAVE ), 1, wavep->fp ) < 1 ||
	     fwrite( RLWAVE_LABEL_FMT,  strlen( RLWAVE_LABEL_FMT  ), 1, wavep->fp ) < 1 ||
	     fwrite( &fmthead_len,    4, 1, wavep->fp ) < 1 ||
	     fwrite( &wavep->is_pcm,  2, 1, wavep->fp ) < 1 ||
	     fwrite( &wavep->channel, 2, 1, wavep->fp ) < 1 ||
	     fwrite( &wavep->rate,    4, 1, wavep->fp ) < 1 ||
	     fwrite( &byte_sec,       4, 1, wavep->fp ) < 1 ||
	     fwrite( &byte_dat,       2, 1, wavep->fp ) < 1 ||
	     fwrite( &wavep->bits,    2, 1, wavep->fp ) < 1 ||
	     fwrite( &shortval,       2, 1, wavep->fp ) < 1 ||
	     fwrite( RLWAVE_LABEL_DATA, strlen( RLWAVE_LABEL_DATA ), 1, wavep->fp ) < 1 ||
	     fwrite( &wavep->len,     4, 1, wavep->fp ) < 1  ) {

		*err = errno;
		return RLIB_ERR;
	}

	return 0;
}


/*!
 * Closes currently opened WAVE file.
 * No error would be returned even if fclose() failed.
 * If "wavep" is NULL, do nothing.
 *
 * @param wavep [i/o] WAVE data struct.
 */
void
rlib_wave_close( RLWAVE* wavep )
{
	if ( wavep != NULL && wavep->fp != NULL ) {
		fclose( wavep->fp );
	}
}


static int 
rlib_wave_check_type( RLWAVE* wavep, int* err )
{
	char buf[8];

	*err = -1;

	/* Reads 4 bytes for "RIFF" and 4 bytes for skip */
	if ( fread( buf, 8, 1, wavep->fp ) < 1 ) {
		*err = errno;

		if ( feof( wavep->fp ) ) {
			return RLIB_EOF;
		}

		return RLIB_ERR;
	}

	if ( strncmp( buf, "RIFF", 4 ) != 0 ) {
		return RLIB_EFMT;
	}

	/* Reads 4 byte for "WAVE" */
	if ( fread( buf, 4, 1, wavep->fp ) < 1 ) {
		*err = errno;

		if ( feof( wavep->fp ) ) {
			return RLIB_EOF;
		}

		return RLIB_ERR;
	}

	if ( strncmp( buf, "WAVE", 4 ) != 0 ) {
		return RLIB_EFMT;
	}

	return 0;
}


static int 
rlib_wave_read_format( RLWAVE* wavep, int* err )
{
	char buf[32];
	int  len;

	*err = -1;

	/* Finds fmt chunk */
	while ( 1 ) {
		if ( fread( buf, 8, 1, wavep->fp ) < 1 ) {
			*err = errno;

			if ( feof( wavep->fp ) ) {
				return RLIB_EOF;
			}

			return RLIB_ERR;
		}

		len = *( ( int* )( &buf[4] ) );

		if ( strncmp( buf, "fmt ", 4 ) != 0 ) {
			/* skips chunk body */
			if ( fseek( wavep->fp, len, SEEK_CUR ) == -1 ) {
				*err = errno;
				return RLIB_ERR;
			}
		}
		else {
			break;
		}
	}


	/* Reads format information */
	if ( fread( buf, len, 1, wavep->fp ) < 1 ) {
		*err = errno;

		if ( feof( wavep->fp ) ) {
			return RLIB_EOF;
		}

		return RLIB_ERR;
	}

	wavep->is_pcm  = *( ( short* )( &buf[0]  ) );
	wavep->channel = *( ( short* )( &buf[2]  ) );
	wavep->rate    = *( ( int*   )( &buf[4]  ) );
	wavep->bits    = *( ( short* )( &buf[14] ) );

	if ( wavep->is_pcm != 1 ) {
		wavep->is_pcm = 0;
	}

	return 0;
}


static int 
rlib_wave_find_pcm( RLWAVE* wavep, int* err )
{
	char buf[8];
	int  len;

	*err = -1;

	/* Finds data chunk */
	while ( 1 ) {
		if ( fread( buf, 8, 1, wavep->fp ) < 1 ) {
			*err = errno;

			if ( feof( wavep->fp ) ) {
				return RLIB_EOF;
			}

			return RLIB_ERR;
		}

		len = *( int* )( &buf[4] );

		if ( strncmp( buf, "data", 4 ) != 0 ) {
			/* skips chunk body */
			if ( fseek( wavep->fp, len, SEEK_CUR ) == -1 ) {
				*err = errno;
				return RLIB_ERR;
			}
		}
		else {
			break;
		}
	}

	wavep->len = len;

	if ( ( wavep->offset = ftell( wavep->fp ) ) == -1 ) {
		*err = errno;
		return RLIB_ERR;
	}

	return 0;
}


/* End of RLwave.c */
