/*
 * Copyright (c) 2005,2007 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief X application tools ( drawing functions ).
 *
 * $Id: RLxdrawing.c,v 1.2 2007/11/04 11:41:12 ryo Exp $
 */

#include <X11/Xlib.h>

#include "RLerrno.h"
#include "RLx.h"


#define RL_X_MAX_RGB 65535


/*!
 * Sets foreground color which will be used when draws something on screen.
 * Specify RGB value as r (RED), g (GREEN), b (BLUE).
 * "scale" must be the maximum value that can be specified to r/g/b. 
 * In this function, r/g/b value will be scaled up or down to 65535 
 * maximum using "scale" value.
 */
void
rlib_x_set_foreground_color( 
	RLXApp* app, int scale, unsigned int r, unsigned int g, unsigned int b )
{
	XColor color;

	if ( app == NULL ) {
		return;
	}

	if ( scale == 0 ) {
		color.red   = r;
		color.green = g;
		color.blue  = b;
	}
	else {
		color.red   =  r * ( RL_X_MAX_RGB / scale );
		color.green =  g * ( RL_X_MAX_RGB / scale );
		color.blue  =  b * ( RL_X_MAX_RGB / scale );
	}

	if ( ! XAllocColor( app->display, app->cmap, &color ) ) {
		return;
	}

	XSetForeground( app->display, app->gc, color.pixel );
}


/*!
 * Sets background color which will be used to draws something on screen.
 * Specify RGB value as r (RED), g (GREEN), b (BLUE).
 * "scale" must be the maximum value that can be specified to r/g/b. 
 * In this function, r/g/b value will be scaled up or down to 65535 
 * maximum using "scale" value.
 */
void
rlib_x_set_background_color( 
	RLXApp* app, int scale, unsigned int r, unsigned int g, unsigned int b )
{
	XColor color;

	if ( app == NULL ) {
		return;
	}

	if ( scale == 0 ) {
		color.red   = r;
		color.green = g;
		color.blue  = b;
	}
	else {
		color.red   =  r * ( RL_X_MAX_RGB / scale );
		color.green =  g * ( RL_X_MAX_RGB / scale );
		color.blue  =  b * ( RL_X_MAX_RGB / scale );
	}

	if ( ! XAllocColor( app->display, app->cmap, &color ) ) {
		return;
	}

	XSetBackground( app->display, app->gc, color.pixel );
}


/*!
 * Draws point at the specified position.
 *
 * @param x [in] x position.
 * @param y [in] y position.
 */
void
rlib_x_draw_point( RLXApp* app, int x, int y )
{
	if ( app != NULL ) {
		XDrawPoint( app->display, app->backbuffer, app->gc, x, y );
	}
}


/*!
 * Draws line from ( src_x, src_y ) to ( dst_x, dst_y ).
 *
 * @param src_x [in] source x position.
 * @param src_y [in] source y position.
 * @param dst_x [in] destination x position.
 * @param dst_y [in] destination y position. 
 */
void
rlib_x_draw_line( RLXApp* app, int src_x, int src_y, int dst_x, int dst_y )
{
	if ( app != NULL ) {
		XDrawLine( 
			app->display, 
			app->backbuffer, 
			app->gc,
			src_x, 
			src_y,
			dst_x,
			dst_y );
	}
}


/*!
 * Draws lines between specified points.
 *
 * @param points  [in] Point data is stored. All points must be specified
 *                     with absolute position correspond to the window,
 *                     not relative position to adjacent points.
 *
 * @param npoints [in] Number of points.
 */
void
rlib_x_draw_lines( RLXApp* app, XPoint* points, int npoints )
{
	if ( app != NULL && points != NULL && npoints >= 0 ) {
		XDrawLines( 
			app->display, 
			app->backbuffer, 
			app->gc, 
			points, 
			npoints, 
			CoordModeOrigin );
	}
}


/*!
 * Fill polygon.
 *
 * @param points  [in] Point data is stored. All points must be specified
 *                     with absolute position correspond to the window,
 *                     not relative position to adjacent points.
 *
 * @param npoints [in] Number of points.
 * @param mode    [in] Specify one of the following. 
 *                     Convex | Nonconvex | Complex.
 *                     If any other value is specified, Convex will be used.
 */
void
rlib_x_fill_polygon( RLXApp* app, XPoint* points, int npoints, int mode )
{
	if ( app != NULL && points != NULL && npoints >= 0 ) {
		int _mode;

		if ( mode != Convex && mode != Nonconvex && mode != Complex ) {
			_mode = Convex;
		}
		else {
			_mode = mode;
		}

		XFillPolygon( 
			app->display, 
			app->backbuffer, 
			app->gc, 
			points, 
			npoints, 
			_mode,
			CoordModeOrigin );
	}
}


/*!
 * Draws rectangle at the specified left-top position and size.
 *
 * @param x      [in] left-top x position.
 * @param y      [in] left-top y position.
 * @param width  [in] width  of the rectangle.
 * @param height [in] height of the rectangle.
 */
void
rlib_x_draw_rectangle( RLXApp* app, int x, int y, int width, int height )
{
	if ( app != NULL ) {
		XDrawRectangle( 
			app->display,
			app->backbuffer,
			app->gc,
			x,
			y,
			( unsigned int )width,
			( unsigned int )height
			);
	}
}


/*!
 * Draws and fills rectangle at the specified left-top position and size.
 *
 * @param x      [in] left-top x position.
 * @param y      [in] left-top y position.
 * @param width  [in] width  of the rectangle.
 * @param height [in] height of the rectangle.
 */
void
rlib_x_fill_rectangle( RLXApp* app, int x, int y, int width, int height )
{
	if ( app != NULL ) {
		XFillRectangle( 
			app->display,
			app->backbuffer,
			app->gc,
			x,
			y,
			( unsigned int )width,
			( unsigned int )height
			);
	}
}


/*!
 * Draws arc within the specified rectangle.
 * Angle of arc is specified with angle1 and angle2, these values
 * must be specified by degrees * 64 ( ex. 360 degrees => 360 x 64 ).
 *
 * @param x      [in] left-top x position of exterior rectangle.
 * @param y      [in] left-top y position of exterior rectangle.
 * @param width  [in] width  of exterior rectangle.
 * @param height [in] height of exterior rectangle.
 * @param angle1 [in] Start angle of arc.
 * @param angle2 [in] End angle of arc.
 */
void
rlib_x_draw_arc( RLXApp* app, int x, int y, int width, int height, int angle1, int angle2 )
{
	if ( app != NULL ) {
		XDrawArc(
			app->display,
			app->backbuffer,
			app->gc,
			x,
			y,
			width,
			height,
			angle1,
			angle2
			);
	}
}


/*!
 * Draws and fills arc within the specified rectangle.
 * Angle of arc is specified with angle1 and angle2, these values
 * must be specified by degrees * 64 ( ex. 360 degrees => 360 x 64 ).
 *
 * @param x      [in] left-top x position of exterior rectangle.
 * @param y      [in] left-top y position of exterior rectangle.
 * @param width  [in] width  of exterior rectangle.
 * @param height [in] height of exterior rectangle.
 * @param angle1 [in] Start angle of arc.
 * @param angle2 [in] End angle of arc.
 */
void
rlib_x_fill_arc( RLXApp* app, int x, int y, int width, int height, int angle1, int angle2 )
{
	if ( app != NULL ) {
		XFillArc(
			app->display,
			app->backbuffer,
			app->gc,
			x,
			y,
			width,
			height,
			angle1,
			angle2
			);
	}
}


/* End of RLxdrawing.c */
