/*
 * Copyright (c) 2005,2006 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief X application tools ( event related functions ).
 *
 * $Id: RLxevent.c,v 1.1.1.1 2006/07/22 18:07:56 ryo Exp $
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "RLerrno.h"
#include "RLlist.h"
#include "RLx.h"


/*!
 * Gets X event saved into the specified application context.
 *
 * @param app [in] GUI toolkit context.
 * @retval X event saved at a certain frame.
 */
XEvent*
rlib_x_get_event( RLXApp* app )
{
	if ( app == NULL ) {
		return NULL;
	}

	if ( app->event_renewed ) {
		return &( app->event );
	}
	else {
		return NULL;
	}
}


/*!
 * Checks if the specified key is pressed at a certain frame.
 * @retval 1 when the specified key is pressed, 0 when not.
 */
int
rlib_x_key_pressed( RLXApp* app, KeySym keysym )
{
	if ( app == NULL ) {
		return 0;
	}

	if ( app->event_renewed && app->event.type == KeyPress ) {
		KeySym pressed_key;
		XLookupString( ( XKeyEvent* )( &( app->event ) ), NULL, 0, &pressed_key, NULL );

		if ( keysym == pressed_key ) {
			return 1;
		}
	}

	return 0;
}


/*!
 * Checks if the specified key is released at a certain frame.
 * @retval 1 when the specified key is released, 0 when not.
 */
int
rlib_x_key_released( RLXApp* app, KeySym keysym )
{
	if ( app == NULL ) {
		return 0;
	}

	if ( app->event_renewed && app->event.type == KeyRelease ) {
		KeySym released_key;
		XLookupString( ( XKeyEvent* )( &( app->event ) ), NULL, 0, &released_key, NULL );

		if ( keysym == released_key ) {
			return 1;
		}
	}

	return 0;
}


/*!
 * Checks if the specified mouse button is pressed at a certain frame.
 *
 * @param app    [in]  GUI toolkit context. 
 * @param button [in]  X mouse button code.
 * @param x      [out] Pressed position will be saved if not NULL.
 * @param y      [out] Pressed position will be saved if not NULL.
 *
 * @retval 1 when specified mouse button is pressed, 0 when not.
 */
int
rlib_x_mouse_pressed( RLXApp* app, unsigned int button, int* x, int* y )
{
	if ( app == NULL ) {
		return 0;
	}

	if ( app->event_renewed && app->event.type == ButtonPress ) {
		XButtonEvent* event = ( XButtonEvent* )( &( app->event ) );

		if ( event->button == button ) {
			if ( x != NULL ) {
				*x = event->x;
			}

			if ( y != NULL ) {
				*y = event->y;
			}

			return 1;
		}
	}

	return 0;
}


/*!
 * Checks if the specified mouse button is released at a certain frame.
 *
 * @param app    [in]  GUI toolkit context.
 * @param button [in]  X mouse button code.
 * @param x      [out] Released position will be saved if not NULL.
 * @param y      [out] Released position will be saved if not NULL.
 *
 * @retval 1 when the specified mouse button is released, 0 when not.
 */
int
rlib_x_mouse_released( RLXApp* app, unsigned int button, int* x, int* y )
{
	if ( app == NULL ) {
		return 0;
	}

	if ( app->event_renewed && app->event.type == ButtonRelease ) {
		XButtonEvent* event = ( XButtonEvent* )( &( app->event ) );

		if ( event->button == button ) {
			if ( x != NULL ) {
				*x = event->x;
			}

			if ( y != NULL ) {
				*y = event->y;
			}

			return 1;
		}
	}

	return 0;
}


/*!
 * Checks if mouse is moved at a certain frame.
 *
 * @param app [in]  GUI toolkit context.
 * @param x   [out] Current position will be saved if not NULL.
 * @param y   [out] Current position will be saved if not NULL.
 *
 * @retval 1 when mouse is moved, 0 when not.
 */
int
rlib_x_mouse_moved( RLXApp* app, int* x, int* y )
{
	if ( app != NULL && app->event_renewed && app->event.type == MotionNotify ) {
		rlib_x_mouse_query_position( app, x, y );
		return 1;
	}

	return 0;
}


/*!
 * Gets current mouse position.
 *
 * @param app [in]  GUI toolkit context.
 * @param x   [out] Current position will be saved if not NULL.
 * @param y   [out] Current position will be saved if not NULL.
 */
void
rlib_x_mouse_query_position( RLXApp* app, int* x, int* y )
{
	if ( app != NULL || x != NULL || y != NULL ) {
		Window root_return;
		Window child_return;
		int root_x_return;
		int root_y_return;
		unsigned int mask_return;

		XQueryPointer( 
			app->display,
			app->frontbuffer,
			&root_return,
			&child_return,
			&root_x_return,
			&root_y_return,
			x,
			y,
			&mask_return );
	}
}


/*!
 * Adds event handler that will be called when specified event occures.
 * When event handler is called, a pointer that represents RLXApp
 * will be passed as an argument.
 *
 * @param app  [i/o] GUI toolkit context.
 * @param type [in]  Event type. Specify one of the following value.
 *
 *  - KeyPress
 *  - KeyRelease
 *  - MotionNotify
 *  - ButtonPress
 *  - ButtonRelease
 *
 * @param callback [in] Call back function called when specified event occures.
 * @param data     [in] This pointer will be passed to call back function.
 *
 * @retval RLIB_EFAULT app is NULL or callback is NULL, or type is invalid.
 * @retval RLIB_ENOSPC Maximum number of callbacks is already registered.
 */
int
rlib_x_add_event_handler( RLXApp* app, int type, int ( *callback )( void* ), void* data )
{
	if ( app == NULL && callback == NULL ) {
		return RLIB_EFAULT;
	}

	if ( rlib_list_is_full( &( app->event_handlers ) ) ) {
		return RLIB_ENOSPC;
	}

	switch ( type ) {
		case KeyPress:
		case KeyRelease:
		case MotionNotify:
		case ButtonPress:
		case ButtonRelease: {
			RLXEventHandler event_handler;
			event_handler.type = type;
			event_handler.callback = callback;
			event_handler.data = data;
			rlib_list_insert( &( app->event_handlers ), -1, 
			       ( unsigned char* )( &event_handler ), sizeof( RLXEventHandler ) );
			return 0;
		}

		default:
			return RLIB_EFAULT;
	}
}


/*!
 * Clear all event handler registered by rlib_x_add_event_handler().
 * @param app [i/o] GUI toolkit context.
 */
void
rlib_x_clear_event_handler( RLXApp* app )
{
	if ( app != NULL ) {
		rlib_list_clear( &( app->event_handlers ) );
	}
}


/*!
 * Process all registered event handler by rlib_x_add_event().
 *
 * @param app [in] GUI toolkit context.
 * @retval 0 All callback functions returned 0. 
 *           Also returns 0 when app is NULL or no event handler is registered.
 * @retval 1 
 */
int
rlib_x_process_event( RLXApp* app ) 
{
	if ( app == NULL || rlib_list_size( &( app->event_handlers ) ) <= 0 ) {
		return 0;
	}

	{
		int count = rlib_list_size( &( app->event_handlers ) );
		int i;

		for ( i = 0; i < count; i ++ ) {
			RLlistData* list_item = rlib_list_get( &( app->event_handlers ), i );
			RLXEventHandler* event_handler = ( RLXEventHandler* )( list_item->buf );

			if ( app->event_renewed && app->event.type == event_handler->type ) {
				if ( event_handler->callback( event_handler->data ) != 0 ) {
					return 1;
				}
			}
		}

		return 0;
	}
}

/* End of RLxevent.c */
