/*
 * Copyright (c) 2005,2006 Ryo Ichinose
 *
 * This file is part of Space Maze.
 * 
 * Space Maze is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 * 
 * Space Maze is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Space Maze; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*!
 * @file
 * @brief X application tools ( image related functions ).
 *
 * $Id: RLximage.c,v 1.1.1.1 2006/07/22 18:07:56 ryo Exp $
 */

#include <X11/Xlib.h>
#include <Imlib.h>

#include "RLerrno.h"
#include "RLx.h"


/*!
 * Draws pixmap loaded beforehand by rlib_x_load_image() or something.
 * 
 * @param app    [in] GUI toolkit context.
 * @param index  [in] Pixmap index to draw returned by loading function ( as rlib_x_load_image() ).
 * @param src_x  [in] Left-top x position of the source pixmap to draw.
 * @param src_y  [in] Left-top y position of the source pixmap to draw.
 * @param width  [in] Width of the source pixmap to draw. 
 *                    If -1 is passed, original width of the source pixmap will be used.
 * @param height [in] Height of the source pixmap to draw.
 *                    If -1 is passed, original height of the source pixmap will be used.
 * @param dst_x  [in] Left-top x position of the destination.
 * @param dst_y  [in] Left-top y position of the destination.
 *
 * @retval 0           Successfully processed.
 * @retval RLIB_EFAULT app is NULL.
 * @retval RLIB_EINVAL index is out of the range of the 
 *                     loaded pixmap number.
 */
int
rlib_x_draw_pixmap( RLXApp* app, int index, 
		 int src_x, int src_y, 
		 int width, int height, 
		 int dst_x, int dst_y )
{
	if ( app == NULL ) {
		return RLIB_EFAULT;
	}

	if ( index >= app->num_of_pixmaps ) {
		return RLIB_EINVAL;
	}

	if ( app->pixmaps[ index ].use_mask == RL_X_USE_MASK ) {
		XSetClipOrigin( app->display, app->gc_pixmap, dst_x - src_x, dst_y - src_y );
		XSetClipMask( app->display, app->gc_pixmap, app->pixmaps[ index ].mask );
	}
	else {
		XSetClipMask( app->display, app->gc_pixmap, None );
	}

	{
		int _width  = ( width  == -1 ) ? app->pixmaps[ index ].width  : width;
		int _height = ( height == -1 ) ? app->pixmaps[ index ].height : height;

		XCopyArea(
			app->display,
			app->pixmaps[ index ].body,
			app->backbuffer,
			app->gc_pixmap,
			src_x,
			src_y,
			_width,
			_height,
			dst_x,
			dst_y );
	}

	return 0;
}


/*!
 * Loads the specified image file and saves it into application context as pixmap.
 * This function can load images which format is supported by Imlib.
 * When successfully processed, returns index to refer loaded image.
 * Use this index when you call rlib_x_draw_pixmap() / rlib_x_get_pixmap_size().
 *
 * @param app      [i/o] GUI toolkit context.
 * @param filename [in]  Image file name ( full path ).
 * @param use_mask [in]  Specify RL_X_USE_MASK when a mask is available from the image.
 *                       Else, specify RL_X_NO_MASK. 
 *                       RL_X_USE_MASK will be used as default.
 *
 * @retval RLIB_EFAULT app is NULL or filename is NULL.
 * @retval RLIB_ENOSPC Maximum number of pixmaps is already loaded.
 * @retval RLIB_ERR    Failed to process this function.
 */
int
rlib_x_load_image( RLXApp* app, char* filename, int use_mask )
{
	ImlibImage* imlib_image = NULL;
	int result;

	if ( app == NULL || filename == NULL ) {
		return RLIB_EFAULT;
	}

	if ( app->num_of_pixmaps > RL_X_MAX_PIXMAP_NUM ) {
		return RLIB_ENOSPC;
	}

	{
		int index = app->num_of_pixmaps;
		imlib_image = Imlib_load_image( app->imlib_data, filename );

		if ( imlib_image == NULL ) {
			return RLIB_ERR;
		}

		app->pixmaps[ index ].width  = imlib_image->rgb_width;
		app->pixmaps[ index ].height = imlib_image->rgb_height;

		Imlib_render( 
			app->imlib_data,
			imlib_image,
			app->pixmaps[ index ].width,
			app->pixmaps[ index ].height 
			);

		app->pixmaps[ index ].body = Imlib_move_image( app->imlib_data, imlib_image );

		if ( app->pixmaps[ index ].body == 0 ) {
			Imlib_kill_image( app->imlib_data, imlib_image );
			return RLIB_ERR;
		}

		if ( use_mask == RL_X_NO_MASK ) {
			app->pixmaps[ index ].use_mask = RL_X_NO_MASK;
		}
		else {
			app->pixmaps[ index ].use_mask = RL_X_USE_MASK;
			app->pixmaps[ index ].mask = 
				Imlib_move_mask( app->imlib_data, imlib_image );

			if ( app->pixmaps[ index ].mask == 0 ) {
				Imlib_free_pixmap( app->imlib_data, app->pixmaps[ index ].body );
				Imlib_kill_image( app->imlib_data, imlib_image );
				return RLIB_ERR;
			}
		}
	}

	Imlib_kill_image( app->imlib_data, imlib_image );
	result = app->num_of_pixmaps ++;
	return result;
}


/*!
 * Gets the size of pixmap loaded beforehand by rlib_x_load_image().
 *
 * @param app    [in]  GUI toolkit context.
 * @param index  [in]  Pixmap index to draw returned by loading function.
 * @param width  [out] Width of specified pixmap will be saved if not NULL.
 * @param height [out] Height of specified pixmap will be saved if not NULL.
 *
 * @retval 0           Successfully processed.
 * @retval RLIB_EFAULT app is NULL.
 * @retval RLIB_EINVAL index is out of the range of the 
 *                     loaded pixmap number.
 */
int
rlib_x_get_pixmap_size( RLXApp* app, int index, int* width, int* height )
{
	if ( app == NULL ) {
		return RLIB_EFAULT;
	}

	if ( index >= app->num_of_pixmaps ) {
		return RLIB_EINVAL;
	}

	if ( width  != NULL ) {
		*width  = app->pixmaps[ index ].width;
	}

	if ( height != NULL ) {
		*height = app->pixmaps[ index ].height;
	}

	return 0;
}


/* End of RLximage.c */
